/*
This program is confidential and proprietary to MRJ Technology Solutions
and may not be reproduced, published or disclosed to others without
written authorization from MRJ.

Copyright (c) 1998 MRJ Technology Solutions, Inc. All Rights Reserved.

This program is derived from prior work some of which was originally
written as a joint project between the Numerical Aerospace Simulation
(NAS) Systems Division of NASA Ames Research Center and the National
Energy Research Supercomputer Center (NERSC) of Lawrence Livermore
National Laboratory.
*/
/* 
 *
 * qmsg - (PBS) send message to batch job
 *
 * Authors:
 *      Terry Heidelberg
 *      Livermore Computing
 *
 *      Bruce Kelly
 *      National Energy Research Supercomputer Center
 *
 *      Lawrence Livermore National Laboratory
 *      University of California
 */

#include "cmds.h"
#include <pbs_config.h>   /* the master config generated by configure */

static char ident[] = "@(#) $RCSfile: qmsg.c,v $ $Revision: 2.0 $";

main(argc, argv, envp) /* qmsg */
int argc;
char **argv;
char **envp;
{
    int c;
    char *errmsg;
    int to_file;
    int errflg=0;
    int any_failed=0;
    struct attrl *attrib = NULL;

    char job_id[PBS_MAXCLTJOBID];       /* from the command line */

    char job_id_out[PBS_MAXCLTJOBID];
    char server_out[MAXSERVERNAME];
    char rmt_server[MAXSERVERNAME];

#define MAX_MSG_STRING_LEN 256
    char msg_string[MAX_MSG_STRING_LEN+1];

#define GETOPT_ARGS "EO"
                 
    msg_string[0]='\0';
    to_file = 0;

    while ((c = getopt (argc, argv, GETOPT_ARGS )) != EOF)
        switch (c) {
        case 'E':
            to_file |= MSG_ERR;
            break;
        case 'O':
            to_file |= MSG_OUT;
            break;
        default :
            errflg++;
        }
    if ( to_file == 0 ) to_file = MSG_ERR;	/* default */

    if (errflg || ((optind+1) >= argc)) {
        static char usage[]=
            "usage: qmsg [-O] [-E] msg_string job_identifier...\n";
        fprintf(stderr, usage);
        exit (2);
    }

    strcpy(msg_string, argv[optind]);
    
    for ( optind++; optind < argc; optind++) {
        int connect;
        int stat=0;
        int stat_msgjob=0;
	int located = FALSE;

        strcpy(job_id, argv[optind]);
        if ( get_server(job_id, job_id_out, server_out) ) {
            fprintf(stderr, "qmsg: illegally formed job identifier: %s\n", job_id);
            any_failed = 1;
            continue;
        }
cnt:
        connect = cnt2server(server_out);
        if ( connect <= 0 ) {
            fprintf(stderr, "qmsg: cannot connect to server %s (errno=%d)\n",
                    pbs_server, pbs_errno);
            any_failed = pbs_errno;
            continue;
        }

        stat = pbs_msgjob(connect, job_id_out, to_file, msg_string, NULL);
        if ( stat && (pbs_errno != PBSE_UNKJOBID) ) {
	    prt_job_err("qmsg", connect, job_id_out);
            any_failed = pbs_errno;
        } else if ( stat && (pbs_errno == PBSE_UNKJOBID) && !located ) {
	    located = TRUE;
	    if ( locate_job(job_id_out, server_out, rmt_server) ) {
	        pbs_disconnect(connect);
		strcpy(server_out, rmt_server);
		goto cnt;
	    }
	    prt_job_err("qmsg", connect, job_id_out);
	    any_failed = pbs_errno;
	}

        pbs_disconnect(connect);
    }
    exit(any_failed);
}
