/*
This program is confidential and proprietary to MRJ Technology Solutions
and may not be reproduced, published or disclosed to others without
written authorization from MRJ.

Copyright (c) 1998 MRJ Technology Solutions, Inc. All Rights Reserved.

This program is derived from prior work some of which was originally
written as a joint project between the Numerical Aerospace Simulation
(NAS) Systems Division of NASA Ames Research Center and the National
Energy Research Supercomputer Center (NERSC) of Lawrence Livermore
National Laboratory.
*/
/*
 * list_link.h - header file for general linked list routines
 *		see list_link.c
 *
 *	A user defined linked list can be managed by these routines if
 *	the first element of the user structure is the list_link struct
 *	defined below.
 */

#ifndef LIST_LINK_H
#define LIST_LINK_H 1
/* list entry list sub-structure */

typedef struct list_link {
	struct list_link *ll_prior;
	struct list_link *ll_next;
	void		 *ll_struct;
} list_link;
typedef list_link list_head;


/* macros to clear list head or link */

#define CLEAR_HEAD(e) e.ll_next = &e, e.ll_prior = &e, e.ll_struct = (void *)0
#define CLEAR_LINK(e) e.ll_next = &e, e.ll_prior = &e

#define LINK_INSET_BEFORE 0
#define LINK_INSET_AFTER  1

#ifdef NDEBUG
#define GET_NEXT(pe)  (pe).ll_next->ll_struct
#define GET_PRIOR(pe) (pe).ll_prior->ll_struct
#else
#define GET_NEXT(pe) get_next(pe, __FILE__, __LINE__)
#define GET_PRIOR(pe) get_prior(pe, __FILE__, __LINE__)
#endif

/* function prototypes */

extern void insert_link A_((list_link *old,list_link *new,void *pobj, int pos));
extern void append_link A_((list_head *head, list_link *new, void *pnewobj));
extern void delete_link A_((list_link *old));
extern void swap_link   A_((list_link *, list_link *));
extern int  is_linked A_((list_link *head, list_link *old));
extern void list_move A_((list_head *old, list_head *new));

#ifndef NDEBUG
extern void *get_next A_((list_link, char *file, int line));
extern void *get_prior A_((list_link, char *file, int line));
#endif	/* NDEBUG */

#endif /* LIST_LINK_H */
