/*
This program is confidential and proprietary to MRJ Technology Solutions
and may not be reproduced, published or disclosed to others without
written authorization from MRJ.

Copyright (c) 1998 MRJ Technology Solutions, Inc. All Rights Reserved.

This program is derived from prior work some of which was originally
written as a joint project between the Numerical Aerospace Simulation
(NAS) Systems Division of NASA Ames Research Center and the National
Energy Research Supercomputer Center (NERSC) of Lawrence Livermore
National Laboratory.
*/
/*
 * Server Work Tasks
 *
 * This structure is used by the server to track deferred work tasks.
 *
 * This information need not be preserved.
 *
 * Other Required Header Files
 *	"list_link.h"
 */

#ifndef WORK_TASK_H
#define WORK_TASK_H 1

enum work_type {
	WORK_Immed,		/* immediate action: see state */
	WORK_Timed,		/* action at certain time */
	WORK_Deferred_Child,	/* On Death of a Child */
	WORK_Deferred_Reply,	/* On reply to an outgoing service request */
	WORK_Deferred_Local,	/* On reply to a local service request */
	WORK_Deferred_Other,	/* various other events */

	WORK_Deferred_Cmp	/* Never set directly, used to indicate that */
				/* a WORK_Deferred_Child is ready            */
};

struct work_task  {
	list_link	 wt_linkall;	/* link to event type work list */
	list_link	 wt_linkobj;	/* link to others of same object */
	long		 wt_event;	/* event id: time, pid, socket, ... */
	enum work_type	 wt_type;	/* type of event */
	void	       (*wt_func) A_((struct work_task *));
					/* function to perform task */
	void		*wt_parm1;	/* obj pointer for use by func */
	void		*wt_parm2;	/* optional pointer for use by func */
	int		 wt_aux;	/* optional info: e.g. child status */
};

extern struct work_task *set_task A_((enum work_type, long event, void (*func)(), void *param));
extern void clear_task A_((struct work_task *ptask));
extern void dispatch_task A_((struct work_task *));

#endif	/* WORK_TASK_H */
