/*
This program is confidential and proprietary to MRJ Technology Solutions
and may not be reproduced, published or disclosed to others without
written authorization from MRJ.

Copyright (c) 1998 MRJ Technology Solutions, Inc. All Rights Reserved.

This program is derived from prior work some of which was originally
written as a joint project between the Numerical Aerospace Simulation
(NAS) Systems Division of NASA Ames Research Center and the National
Energy Research Supercomputer Center (NERSC) of Lawrence Livermore
National Laboratory.
*/
#include <pbs_config.h>   /* the master config generated by configure */

#include <assert.h>
#include <ctype.h>
#include <memory.h>
#ifndef NDEBUG
#include <stdio.h>
#endif
#include <stdlib.h>
#include <string.h>
#include "pbs_ifl.h"
#include "list_link.h"
#include "attribute.h"
#include "pbs_error.h"

static char ident[] = "@(#) $RCSfile: attr_fn_c.c,v $ $Revision: 2.0 $";

/*
 * This file contains functions for manipulating attributes
 *	character, a single
 *
 * Each set has functions for:
 *	Decoding the value string to the machine representation.
 *	Encoding the machine representation of the attribute to external form
 *	Setting the value by =, + or - operators.
 *	Comparing a (decoded) value with the attribute value.
 *
 * Some or all of the functions for an attribute type may be shared with
 * other attribute types.
 * 
 * The prototypes are declared in "attribute.h"
 *
 * --------------------------------------------------
 * The Set of Attribute Functions for attributes with
 * value type "char"
 * --------------------------------------------------
 */

/*
 * decode_c - decode first character of string into attribute structure
 *
 *	Returns: 0 if ok
 *		>0 error number if error
 *		*patr elements set
 */

int decode_c(patr, name, rescn, val)
	struct attribute *patr;
	char *name;		/* attribute name */
	char *rescn;		/* resource name, unused here */
	char *val;		/* attribute value */
{
	if ((val == (char *)0) || (strlen(val) == 0)) {
		patr->at_flags = (patr->at_flags & ~ATR_VFLAG_SET) |
				 ATR_VFLAG_MODIFY;
		patr->at_val.at_char = '\0';
	} else {
		patr->at_flags |= ATR_VFLAG_SET | ATR_VFLAG_MODIFY;
		patr->at_val.at_char = *val;
	}
	return (0);
}

/*
 * encode_c - encode attribute of type character into attr_extern
 *
 *	Returns: >0 if ok
 *		 =0 if no value, no link entry created
 *		 <0 if error
 */
/*ARGSUSED*/

int encode_c(attr, phead, atname, rsname, mode)
	attribute	*attr;		/* ptr to attribute */
	list_head	*phead;		/* head of attrlist list */
	char		*atname;	/* attribute name */
	char		*rsname;	/* resource name or null */
	int		mode;		/* encode mode, unused here */
{

	svrattrl *pal;

	if ( !attr )
		return (-1);
	if ( !(attr->at_flags & ATR_VFLAG_SET))
		return (0);
	
	pal = attrlist_create(atname, rsname, 2);
	if (pal == (svrattrl *)0)
		return (-1);

	*pal->al_value   = attr->at_val.at_char;
	*(pal->al_value+1) = '\0';
	pal->al_flags = attr->at_flags;
	append_link(phead, &pal->al_link, pal);

	return (1);
}

/*
 * set_c - set attribute A to attribute B,
 *	either A=B, A += B, or A -= B
 *
 *	Returns: 0 if ok
 *		>0 if error
 */

int set_c(attr, new, op)
	struct attribute *attr;
	struct attribute *new;
	enum batch_op op;
{
	assert( attr && new && (new->at_flags & ATR_VFLAG_SET));

	switch (op) {
	    case SET:	attr->at_val.at_char = new->at_val.at_char;
			break;

	    case INCR:	attr->at_val.at_char = 
			    (char)((int)attr->at_val.at_char +
			           (int)new->at_val.at_char);
			break;

	    case DECR:	attr->at_val.at_char = 
			    (char)((int)attr->at_val.at_char -
				   (int)new->at_val.at_char);
			break;

	    default:	return (PBSE_INTERNAL);
	}
	attr->at_flags |= ATR_VFLAG_SET | ATR_VFLAG_MODIFY;
	return (0);
}

/*
 * comp_c - compare two attributes of type character
 *
 *	Returns: +1 if 1st > 2nd
 *		  0 if 1st == 2nd
 *		 -1 if 1st < 2nd
 */

int comp_c(attr, with)
	struct attribute *attr;
	struct attribute *with;
{
	if ( !attr || !with )
		return (-1);
	if (attr->at_val.at_char < with->at_val.at_char)
		return (-1);
	else if (attr->at_val.at_char >  with->at_val.at_char)
		return (1);
	else
		return (0);
}

/*
 * free_c - use free_null() to (not) free space
 */

