/*
This program is confidential and proprietary to MRJ Technology Solutions
and may not be reproduced, published or disclosed to others without
written authorization from MRJ.

Copyright (c) 1998 MRJ Technology Solutions, Inc. All Rights Reserved.

This program is derived from prior work some of which was originally
written as a joint project between the Numerical Aerospace Simulation
(NAS) Systems Division of NASA Ames Research Center and the National
Energy Research Supercomputer Center (NERSC) of Lawrence Livermore
National Laboratory.
*/
#include <pbs_config.h>   /* the master config generated by configure */

#include <assert.h>
#include <ctype.h>
#include <memory.h>
#ifndef NDEBUG
#include <stdio.h>
#endif
#include <stdlib.h>
#include <string.h>
#include "pbs_ifl.h"
#include "list_link.h"
#include "attribute.h"
#include "pbs_error.h"

static char ident[] = "@(#) $RCSfile: attr_fn_intr.c,v $ $Revision: 2.0 $";

/*
 * This file contains functions for manipulating attributes of type
 *	interactive
 *
 * Each set has functions for:
 *	Decoding the value string to the machine representation.
 *	Encoding the machine representation of the value to a string
 *	Setting the value by =, + or - operators.
 *	Comparing a (decoded) value with the attribute value.
 *	Freeing the space malloc-ed to the attribute value.
 *
 * Some or all of the functions for an attribute type may be shared with
 * other attribute types.
 * 
 * The prototypes are declared in "attribute.h"
 *
 * -----------------------------------------------------------
 * Set of General functions for attributes of type interactive
 * -----------------------------------------------------------
 *
 * This attribute contains the port number to which an Interactive qsub is
 * listening.  
 */


/* decode_interactive - use decode_l() */

/*
 * encode_inter - encode attribute of type ATR_TYPE_STR to attr_extern
 *
 *	Special case for "interactive" attribute,  decode into TRUE/FALSE
 *	for  client, decode into port number for all others.
 *
 *	Returns: >0 size of data returned to buffer
 *		 =0 if not value, no link added
 *		 <0 if error
 */
/*ARGSUSED*/

int encode_inter(attr, phead, atname, rsname, mode)
	attribute	*attr;		/* ptr to attribute */
	list_head	*phead;		/* head of attribute list */
	char		*atname;	/* attribute name */
	char		*rsname;	/* resource name or null */
	int		 mode;		/* encode mode, unused here */
{
	if (mode == ATR_ENCODE_CLIENT)
		return (encode_b(attr, phead, atname, rsname, mode));
	else
		return (encode_l(attr, phead, atname, rsname, mode));
}
