/*
This program is confidential and proprietary to MRJ Technology Solutions
and may not be reproduced, published or disclosed to others without
written authorization from MRJ.

Copyright (c) 1998 MRJ Technology Solutions, Inc. All Rights Reserved.

This program is derived from prior work some of which was originally
written as a joint project between the Numerical Aerospace Simulation
(NAS) Systems Division of NASA Ames Research Center and the National
Energy Research Supercomputer Center (NERSC) of Lawrence Livermore
National Laboratory.
*/
#include <pbs_config.h>   /* the master config generated by configure */

#include <limits.h>
#include <assert.h>
#include <ctype.h>
#include <memory.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <pbs_ifl.h>
#include "list_link.h"
#include "attribute.h"
#include "pbs_error.h"

static char ident[] = "@(#) $RCSfile: attr_fn_time.c,v $ $Revision: 2.0 $";

/*
 * This file contains functions for manipulating attributes of type
 *	time:	[[hh:]mm:]ss[.sss]
 *
 * Each set has functions for:
 *	Decoding the value string to the machine representation.
 *	Encoding the internal attribute to external form
 *	Setting the value by =, + or - operators.
 *	Comparing a (decoded) value with the attribute value.
 *
 * Some or all of the functions for an attribute type may be shared with
 * other attribute types.
 * 
 * The prototypes are declared in "attribute.h"
 *
 * --------------------------------------------------
 * The Set of Attribute Functions for attributes with
 * value type "long"
 * --------------------------------------------------
 */


/*
 * decode_time - decode time into into attribute structure of type ATR_TYPE_LONG
 *
 *	Returns: 0 if ok
 *		>0 error number if error
 *		*patr elements set
 */

#define PBS_MAX_TIME (LONG_MAX - 1)
int decode_time(patr, name, rescn, val)
	struct attribute *patr;
	char *name;		/* attribute name */
	char *rescn;		/* resource name, unused here */
	char *val;		/* attribute value */
{
	int   i;
	char  msec[4];
	int   ncolon = 0;
	char *pc;
	long  rv = 0;
	char *workval;
	char *workvalsv;

	if ((val == (char *)0) || (strlen(val) == 0)) {
		patr->at_flags = (patr->at_flags & ~ATR_VFLAG_SET) |
				 ATR_VFLAG_MODIFY;
		patr->at_val.at_long = 0;
		return (0);
	}

	workval = strdup(val);
	workvalsv = workval;

	for (i = 0; i < 3; ++i)
		msec[i] = '0';
	msec[i] = '\0';

	for (pc = workval; *pc; ++pc) {

		if (*pc == ':') {
			if (++ncolon > 2)
				goto badval;
			*pc = '\0';
			rv = (rv * 60) + atoi(workval);
			workval = pc + 1;

		} else if (*pc == '.') {
			*pc++ = '\0';
			for (i = 0; (i < 3) && *pc; ++i)
				msec[i] = *pc++;
			break;
		} else if ( !isdigit((int)*pc) ) { 
			goto badval;	/* bad value */
		}
	}
	rv = (rv * 60) + atoi(workval);
	if (rv > PBS_MAX_TIME)
		goto badval;
	if (atoi(msec) >= 500)
		rv++;
	patr->at_val.at_long = rv;
	patr->at_flags |= ATR_VFLAG_SET | ATR_VFLAG_MODIFY;
	(void)free(workvalsv);
	return (0);

badval:	(void)free(workvalsv);
	return (PBSE_BADATVAL);
}

/*
 * encode_time - encode attribute of type long into attr_extern
 *	with value in form of [[hh:]mm:]ss
 *
 *	Returns: >0 if ok
 *		 =0 if no value, no attrlist link added
 *		 <0 if error
 */
/*ARGSUSED*/

#define CVNBUFSZ 21

int encode_time(attr, phead, atname, rsname, mode)
	attribute	*attr;	  /* ptr to attribute */
	list_head	*phead;	  /* head of attrlist list */
	char		*atname;  /* attribute name */
	char		*rsname;  /* resource name or null */
	int		mode;	  /* encode mode, unused here */
{
	size_t	  ct;
	char	  cvnbuf[CVNBUFSZ];
	int 	  hr;
	int	  min;
	long	  n;
	svrattrl *pal;
	int	  sec;
	char	 *pv;

	if ( !attr )
		return (-1);
	if ( !(attr->at_flags & ATR_VFLAG_SET))
		return (0);

	n   = attr->at_val.at_long;
	hr  = n / 3600;
	n   = n % 3600;
	min = n / 60;
	n   = n % 60;
	sec = n;

	pv = cvnbuf;
	(void)sprintf(pv, "%02d:%02d:%02d", hr, min, sec);
	pv += strlen(pv);

	ct = strlen(cvnbuf) + 1;

	pal = attrlist_create(atname, rsname, ct);
	if (pal == (svrattrl *)0)
		return (-1);

	(void)memcpy(pal->al_value, cvnbuf, ct);
	pal->al_flags = attr->at_flags;
	append_link(phead, &pal->al_link, pal);
	
	return (1);
}

/*
 * set_time  - use the function set_l()
 *
 * comp_time - use the funttion comp_l()
 *
 * free_l - use free_null to (not) free space
 */
