/*
This program is confidential and proprietary to MRJ Technology Solutions
and may not be reproduced, published or disclosed to others without
written authorization from MRJ.

Copyright (c) 1998 MRJ Technology Solutions, Inc. All Rights Reserved.

This program is derived from prior work some of which was originally
written as a joint project between the Numerical Aerospace Simulation
(NAS) Systems Division of NASA Ames Research Center and the National
Energy Research Supercomputer Center (NERSC) of Lawrence Livermore
National Laboratory.
*/

#include <pbs_config.h>   /* the master config generated by configure */

#include <sys/types.h>
#include <ctype.h>
#include <memory.h>
#ifndef NDEBUG
#include <stdio.h>
#endif
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include "pbs_ifl.h"
#include "list_link.h"
#include "attribute.h"
#include "server_limits.h"
#include "net_connect.h"
#include "job.h"
#include "pbs_nodes.h"
#include "pbs_error.h"

static char ident[] = "@(#) $RCSfile: attr_node_func.c,v $ $Revision: 2.0 $";

/*
 * This file contains functions for deriving attribute values from a pbsnode
 * and for updating the "state" (inuse), "node type" (ntype) or "properties"
 * list using the "value" carried in an attribute.
 *
 * Included are:
 *
 * global:
 * decode_state()		"functions for at_decode func pointer"
 * decode_ntype()
 * decode_props()
 *
 * encode_state()		"functions for at_encode func pointer"
 * encode_ntype()
 * encode_props()
 * encode_jobs()
 *
 * set_node_state()		"functions for at_set func pointer"
 * set_node_ntype()
 * set_node_props()
 *
 * node_state()			"functions for at_action func pointer"
 * node_ntype()
 * node_prop_list()
 *
 * free_prop_list()
 * free_prop_attr()		"function for at_free func pointer"
 *
 * local:
 * load_prop()
 * set_nodeflag()
 *
 * The prototypes are declared in "attr_func.h"
 */


/*
 * Set of forward declarations for functions used before defined
 * keeps the compiler happy
*/
static	int	set_nodeflag A_(( char*, short*));
static	int	load_prop A_(( char*, struct prop* ));


/*
 * encode_state
 * Once the node's "inuse" field is converted to an attribute,
 * the attribute can be passed to this function for encoding into
 * an svrattrl structure
 * Returns  <0       an error encountered; value is negative of an error code
 *           0       ok, encode happened and svrattrl created and linked in,
 *		     or nothing to encode
 */
 
int	encode_state(pattr, ph, aname, rname, mode)
	attribute	 *pattr;	/*struct attribute being encoded  */
	list_head	 *ph;		/*head of a list of  "svrattrl"   */
					/*structs which are to be returned*/

	char		 *aname;	/*attribute's name		  */
	char		 *rname;	/*resource's name (null if none)  */
	int		 mode;		/*mode code, unused here	  */

{
        svrattrl *pal;
	short	 state;
	char	 **pstate;
	static	 char	*ns[] = { ND_free, ND_offline, ND_down, ND_reserve,
				  ND_job_exclusive, ND_job_sharing, ND_state_unknown, "" };

	
	static   char	state_str[ MAX_ENCODE_BFR ];

        if ( !pattr )
		return -(PBSE_INTERNAL);

        if ( !(pattr->at_flags & ATR_VFLAG_SET) )
                return (0);      /*nothing to report back*/
	
	state = pattr->at_val.at_short & PBSNODE_STATE_MASK;
	if (!state)
	    strcpy( state_str, ns[0] );

	else {
	   state_str[0] = '\0';
	   pstate = &ns[1];
	   while( state ) {
		if( state & 0x1 ) {
		    if( *state_str )
		        strcat( state_str, ", " );

		    strcat( state_str, *pstate );
		}
		state >>= 1;
		pstate++;
	   }
	}
	
        pal = attrlist_create(aname,rname,(int)strlen(state_str)+1);
        if (pal == (svrattrl *)0)
                return -(PBSE_SYSTEM);

        (void)strcpy(pal->al_value, state_str);
        pal->al_flags = ATR_VFLAG_SET;
        append_link(ph, &pal->al_link, pal);

        return (0);             /*success*/
}


/*
 * encode_ntype
 * Once the node's "ntype" field is converted to an attribute,
 * the attribute can be passed to this function for encoding into
 * an svrattrl structure
 * Returns  <0       an error encountered; value is negative of an error code
 *           0       ok, encode happened and svrattrl created and linked in,
 *		     or nothing to encode
 */
 
int	encode_ntype(pattr, ph, aname, rname, mode)
	attribute	 *pattr;	/*struct attribute being encoded  */
	list_head	 *ph;		/*head of a list of  "svrattrl"   */
					/*structs which are to be returned*/

	char		 *aname;	/*attribute's name		  */
	char		 *rname;	/*resource's name (null if none)  */
	int		 mode;		/*mode code, unused here	  */

{
        svrattrl *pal;
	short	 ntype;
	static	 char	*nt[] = { ND_cluster, ND_timeshared };

	static   char	ntype_str[ MAX_ENCODE_BFR ];

        if ( !pattr )
                return -(PBSE_INTERNAL);

        if ( !(pattr->at_flags & ATR_VFLAG_SET) )
                return (0);      /*nothing to report back*/
	
	ntype = pattr->at_val.at_short & PBSNODE_NTYPE_MASK;
	if (!ntype)
	    strcpy( ntype_str, nt[0] );

	else
	    strcpy( ntype_str, nt[1] );
	
        pal = attrlist_create(aname,rname,(int)strlen(ntype_str)+1);
        if (pal == (svrattrl *)0)
                return -(PBSE_SYSTEM);

        (void)strcpy(pal->al_value, ntype_str);
        pal->al_flags = ATR_VFLAG_SET;
        append_link(ph, &pal->al_link, pal);

        return (0);             /*success*/
}


/*
 * encode_props
 * Once the node's "properties list" is converted to a "prop" attribute,
 * the attribute can be passed to this function for encoding into
 * an svrattrl structure
 * Returns  <0       an error encountered; value is negative of an error code
 *           0       ok, encode happened and svrattrl created and linked in,
 *		     or nothing to encode
 */
 
int	encode_props(pattr, ph, aname, rname, mode)
	attribute	 *pattr;	/*struct attribute being encoded  */
	list_head	 *ph;		/*head of a  list of "svrattrl"   */
					/*structs which are to be returned*/

	char		 *aname;	/*attribute's name		  */
	char		 *rname;	/*resource's name (null if none)  */
	int		 mode;		/*mode code, unused here	  */

{
        svrattrl	*pal;
	struct prop	*pp;
	int		i;
	int		propcnt;	/*number of props in the list	   */
	int		strsize;	/*computed string size		   */
	char		*prop_str;	/*holds comma separated prop string*/

        if ( !pattr )
                return (-1);

        if ( !(pattr->at_flags & ATR_VFLAG_SET) || !pattr->at_val.at_prop )
                return (0);
	
	/*cnt number of properties and estimate size of string buffer required*/
	propcnt = 0;
	strsize = 1;			     /*space for terminating null char*/
	pp = pattr->at_val.at_prop;
	while( pp ) {
	   propcnt++;
	   strsize += strlen( pp->name ) + 2 /*  ", "  */ ;
	   pp = pp->next;
	}

	if( propcnt == 1 )
	      return (0);		     /*only have the name of the node*/

	else if ( !(prop_str = (char *)malloc( strsize )) )
	      return -(PBSE_SYSTEM);

	prop_str[0] = '\0';
	pp = pattr->at_val.at_prop;
	for( i=0; i<propcnt - 1; i++ ) {     /*don't include node name*/
	     if (i > 0 )
		 strcat(prop_str, ", ");
	     strcat(prop_str, pp->name);	
	     pp = pp->next;
	}

        pal = attrlist_create(aname, rname, (int)strlen(prop_str) + 1  );
        if (pal == (svrattrl *)0)
                return -(PBSE_SYSTEM);

        (void)strcpy(pal->al_value, prop_str);
        pal->al_flags = ATR_VFLAG_SET;
	free( prop_str );

        append_link(ph, &pal->al_link, pal);

        return (0);           /*success*/
}


/*
 * encode_jobs
 * Once the node's struct jobinfo pointer is put in the data area of 
 * temporary attribute, this function will walk the list of jobs and
 * generate the comma separated list to send back via an svrattrl
 * structure.
 * Returns  <0       an error encountered; value is negative of an error code
 *           0       ok, encode happened and svrattrl created and linked in,
 *		     or nothing to encode
 */
 
int	encode_jobs(pattr, ph, aname, rname, mode)
	attribute	 *pattr;	/*struct attribute being encoded  */
	list_head	 *ph;		/*head of a  list of "svrattrl"   */
					/*structs which are to be returned*/

	char		 *aname;	/*attribute's name		  */
	char		 *rname;	/*resource's name (null if none)  */
	int		 mode;		/*mode code, unused here	  */

{
        svrattrl	*pal;
	struct jobinfo	*jip;
	int		i;
	int		jobcnt;		/*number of jobs using the node     */
	int		strsize;	/*computed string size		    */
	char		*job_str;	/*holds comma separated list of jobs*/

        if ( !pattr )
                return (-1);
        if ( !(pattr->at_flags & ATR_VFLAG_SET) || !pattr->at_val.at_jinfop )
                return (0);                  /*nothing to report back   */
	
	/*cnt number of jobs and estimate size of string buffer required*/
	jobcnt = 0;
	strsize = 1;			     /*allow for terminating null char*/
	jip = pattr->at_val.at_jinfop;
	while( jip ) {
	   jobcnt++;
	   strsize += strlen( jip->job->ji_qs.ji_jobid ) + 2 /*  ", "  */ ;
	   jip = jip->next;
	}

	if( jobcnt == 0 )
	      return (0);		     /*no jobs currently on this node*/

	else if ( !(job_str = (char *)malloc( strsize )) )
	      return -(PBSE_SYSTEM);

	job_str[0] = '\0';
	jip = pattr->at_val.at_jinfop;
	for( i=0; i<jobcnt; i++ ) {
	     if (i > 0 )
		 strcat(job_str, ", ");
	     strcat(job_str, jip->job->ji_qs.ji_jobid);	
	     jip = jip->next;
	}

        pal = attrlist_create(aname, rname, (int)strlen(job_str) + 1  );
        if ( pal == (svrattrl *)0 ) {
		free(job_str);
                return -(PBSE_SYSTEM);
	}

        (void)strcpy(pal->al_value, job_str);
        pal->al_flags = ATR_VFLAG_SET;
	free( job_str );

        append_link(ph, &pal->al_link, pal);

        return (0);                  /*success*/
}


/*
 * decode_state 
 * In this case, the two arguments that get  used are
 * pattr-- it points to an attribute whose value is a short,
 * and the argument "val".
 * Once the "value" argument, val, is decoded from its form
 * as a string of comma separated substrings, the component
 * values are used to set the appropriate bits in the attribute's
 * value field.
 */
 
int	decode_state(pattr, name, rescn, val)
        attribute *pattr;
        char *name;             /* attribute name */
        char *rescn;            /* resource name, unused here */
        char *val;              /* attribute value */

{
	int	rc = 0;		/*return code; 0==success*/
	short	flag, currflag;
	char    *str;

	char	strbuf[512];	/*should handle most vals*/
	char	*sbufp;
	int	slen;


	if(val == (char *)0)
		return (PBSE_BADNDATVAL);


        /*
         * determine string storage requirement and copy the string "val"
         * to a work buffer area
         */

	slen = strlen( val );	/*bufr either on stack or heap*/
	if( slen - 512 < 0 )
	    sbufp = strbuf;
	else {
	    if( !(sbufp = (char *)malloc( slen + 1 )) )
		return (PBSE_SYSTEM);
	}

	strcpy( sbufp, val );

	if( (str = parse_comma_string( sbufp )) == (char *)0 ) {
	     if( slen >= 512 )
		 free( sbufp );
	     return rc;
	}

	flag = 0;
	if( rc = set_nodeflag( str, &flag ) ) {
	    if( slen >= 512 )
		 free( sbufp );
	    return rc;
	}
	currflag = flag;


	/*calling parse_comma_string with a null ptr continues where*/
	/*last call left off.  The initial comma separated string   */
	/*copy pointed to by sbufp is modified with each func call  */

	while( str = parse_comma_string( (char *)0 ) ) {
	    if( rc = set_nodeflag( str, &flag ) )
		break;

	    if( (currflag == 0 && flag) || (currflag && flag == 0) ) {
		rc = PBSE_MUTUALEX;	/*free is mutually exclusive*/
		break;
	    }
	    currflag = flag;
	}

	if( !rc )
	{
	    pattr->at_val.at_short = flag;
	    pattr->at_flags |= ATR_VFLAG_SET|ATR_VFLAG_MODIFY;
	}

	if( slen >= 512 )		/*buffer on heap, not stack*/
	    free( sbufp );

	return rc;
}


/*
 * decode_ntype 
 * In this case, the two arguments that get  used are
 * pattr-- it points to an attribute whose value is a short,
 * and the argument "val".
 * Although we have only "time-shared" and "cluster" at this
 * point in PBS's evolution, there may come a time when other
 * ntype values are needed.  The one thing that is assumed is
 * that the types are going to be mutually exclusive
 */
 
int	decode_ntype(pattr, name, rescn, val)
        attribute *pattr;
        char *name;             /* attribute name */
        char *rescn;            /* resource name, unused here */
        char *val;              /* attribute value */

{
	int	rc = 0;		/*return code; 0==success*/
	short	tmp = 0;


	if( val == (char *)0 )
	   rc = (PBSE_BADNDATVAL);
	else if( !strcmp(val, ND_timeshared) )
	   tmp = NTYPE_TIMESHARED;
	else if( !strcmp(val, ND_cluster) )
	   tmp = NTYPE_CLUSTER;
	else
	   rc = (PBSE_BADNDATVAL);


	if( !rc )
	{
	    pattr->at_val.at_short = tmp;
	    pattr->at_flags |= ATR_VFLAG_SET|ATR_VFLAG_MODIFY;
	}

	return rc;
}


/*
 * decode_props - In this case, the two arguments that get used are
 *	       pattr-- which points to a "props" attribute and, "val"
 *	       which is a string of comma separated substrings.  Once
 *	       val's components are decoded into a linked list of "prop"
 *	       structures, this list is hung on pattr's "at_val" field.
 */
 
 
int	decode_props(pattr, name, rescn, val)
        attribute	 *pattr;
        char		 *name;             /* attribute name, unused here */
        char		 *rescn;            /* resource name, unused here  */
        char		 *val;              /* attribute value */

{
	int			 rc = 0;
        int                      ns, i;
        char                    *pbuf;
        char                    *pc;
	struct prop		 dum_prop;
	struct prop		*pp;
	struct prop		*pp1;
	struct prop		*pp2;
	struct prop		*pp3;
	struct prop		*pplast;

	char	strbuf[512];	/*should handle most vals*/
	char	*sbufp;
	int	slen;


        /*
         * determine string storage requirement and copy the string "val"
         * to a work buffer area
         */

	slen = strlen( val );	/*bufr either on stack or heap*/
	if( slen - 512 < 0 )
	    sbufp = strbuf;
	else {
	    if( !(sbufp = (char *)malloc( slen + 1 )) )
		return (PBSE_SYSTEM);
	}

	strcpy( sbufp, val );


        /*
         * determine number of substrings, each substring is terminated
         * by a non-escaped comma or a new-line, the whole string is terminated
         * by a null
         */

	if( rc = count_substrings( sbufp, &ns ) ) {
	    if( slen - 512 >= 0 )
		free( sbufp );
	    return  rc;
	}

	/*get a linked list of prop structs and initialize its components*/
	/*if the list can only be partially acquired, return a nonzero rc*/
	/*caller calls attr_atomic_kill() on nonzero rc to do all cleanup*/

	if( (pp = malloc(sizeof(struct prop))) == (struct prop *)0 ) {
	     if( slen >= 512 )
		 free( sbufp );
	     return (PBSE_SYSTEM);
	}

	pattr->at_val.at_prop = pp;
	pattr->at_val.at_prop->name = (char *)0;
	pattr->at_val.at_prop->mark = 0;
	pattr->at_val.at_prop->next = (struct prop *)0;
	for(i=0; i<ns - 1; i++) {

	    if( (pp = malloc(sizeof(struct prop))) == (struct prop *)0 ) {

	     	 if( slen >= 512 )
		     free( sbufp );
		 return (PBSE_SYSTEM);
	    }

	    pp->name = (char *)0;
	    pp->mark = 0;
	    pp->next = pattr->at_val.at_prop;
	    pattr->at_val.at_prop = pp;
	}

	/*get the properties for the list of prop structures*/
	/*calling load_prop with non-null first argument initializes an internal*/
	/*string pointer.  Calling it with a first argument of zero allows the  */
	/*caller to continue on in the comma separated input string where the   */
	/*previous call stopped.						*/

	pp = pattr->at_val.at_prop;
	if( !(rc = load_prop( sbufp, pp )) ) { /*1st  load_prop returned success*/

	    for( pp = pp->next; pp ; pp = pp->next ) {

		 if ( rc = load_prop( (char *)0, pp ) )
		      break;                              /*failure on load_prop*/
	     }
	}

	if( !rc )
	    pattr->at_flags |= ATR_VFLAG_SET;
	else {

	     if( slen >= 512 )
	         free( sbufp );
	}

	return rc;
}



/*
 * load_prop - malloc buffer space to hold the string pointed to by
 *	       argument, "val".  Copy the string into this new space
 *	       and hang it on the prop struct pointed to by argument, "pp"
 */
static	int	load_prop( val, pp )
	char		*val;
	struct prop	*pp;
{
	int	rc = 0;
	char   *str;
	int	len;

	str = parse_comma_string(val);

	if( !(len = strlen(str)) ) 
	    rc = PBSE_BADNDATVAL;
	else if( (pp->name = malloc(len + 1)) == (char *)0 ) 
	    rc = PBSE_SYSTEM;
	else
	    strcpy( pp->name, str );

	return rc;

}


/*
 * free_prop_attr
 * calls "free_prop_list" to clean up the null terminated prop
 * list pointed to by "pattr->at_val.at_prop".  As part of the
 * "freeing" operation the attribute's "VALUE is SET" flag gets
 * cleared and it's value (a struct prop*) is set to 0
 */

void	free_prop_attr( pattr )
	attribute	*pattr;
{
	struct prop	*pp, *prev;

	if( !pattr->at_val.at_prop )
	    return;
	free_prop_list( pattr->at_val.at_prop );
	pattr->at_flags &= ~ATR_VFLAG_SET;

	/*just freed that to which it pointed, so for safety sake*/
	pattr->at_val.at_prop = (struct prop *)0;
}


/*
 * free_prop_list
 * For each element of a null terminated prop list call free
 * to clean up any string buffer that hangs from the element.
 * After this, call free to remove the struct prop.
 */

void	free_prop_list( prop )
	struct prop	*prop;
{
	struct prop	*pp;

	while( prop ) {
	   if( prop->name )
	       free(prop->name);
	   pp = prop->next;
	   free(prop);
	   prop = pp;
	}
}


/*
 * set_node_state - the information in the "short" attribute, *new, is used to
 *		    update the information in the "short" attribute, *pattr.
 *		    the mode of the update is goverened by the argument "op"
 *		    (SET,INCR,DECR)
 */

int	set_node_state( pattr, new, op)
	attribute	*pattr;		/*attribute gets modified    */
	attribute	*new;		/*carries new, modifying info*/
					/*that got decoded into 'new"*/
	enum batch_op	 op;
{
	int	rc = 0;

	assert( pattr && new && (new->at_flags & ATR_VFLAG_SET) );

	switch( op ) {

	    case SET:
	    pattr->at_val.at_short = new->at_val.at_short;
	    break;

	    case INCR:
	    if( (pattr->at_val.at_short && new->at_val.at_short == 0) ||
		(pattr->at_val.at_short == 0 && new->at_val.at_short)    ) {

			rc = PBSE_BADNDATVAL;  /*"free" mutually exclusive*/
			break;
	    }

	    pattr->at_val.at_short |= new->at_val.at_short;
	    break;

	    case DECR:
	    if( pattr->at_val.at_short && new->at_val.at_short == 0 ) {

		       rc = PBSE_BADNDATVAL;  /*"free" mutually exclusive*/
		       break;
	    }
	    if( pattr->at_val.at_short & new->at_val.at_short
		!= new->at_val.at_short ) {

			rc = PBSE_BADNDATVAL;  /*not subset of existing*/
			break;
	    }

	    pattr->at_val.at_short &= ~new->at_val.at_short;
		   break;
		   default:	rc = PBSE_INTERNAL;
	}

	if ( !rc )
	     pattr->at_flags |= ATR_VFLAG_SET | ATR_VFLAG_MODIFY;
	return rc;
}


/*
 * set_node_ntype - the value entry in attribute "new" is a short.  It was
 *		    generated by the decode routine is used to update the
 *		    value portion of the attribute *pattr
 *		    the mode of the update is goverened by the argument "op"
 *		    (SET,INCR,DECR)
 */

int	set_node_ntype ( pattr, new, op)
	attribute	*pattr;		/*attribute gets modified    */
	attribute	*new;		/*carries new, modifying info*/
					/*that got decoded into 'new"*/
	enum batch_op	 op;
{
	int	rc = 0;

	assert( pattr && new && (new->at_flags & ATR_VFLAG_SET) );

	switch( op ) {

	    case SET:
	    pattr->at_val.at_short = new->at_val.at_short;
	    break;

	    case INCR:
	    if( pattr->at_val.at_short != new->at_val.at_short ) {

			rc = PBSE_MUTUALEX;  /*types are mutually exclusive*/
	    }
	    break;

	    case DECR:
	    if( pattr->at_val.at_short != new->at_val.at_short )
			rc = PBSE_MUTUALEX;  /*types are mutually exclusive*/

	    else if( pattr->at_val.at_short == NTYPE_TIMESHARED )
			pattr->at_val.at_short = NTYPE_CLUSTER;

	    else
			pattr->at_val.at_short = NTYPE_TIMESHARED;

	    break;

	    default:	rc = PBSE_INTERNAL;
	}

	if ( !rc )
	     pattr->at_flags |= ATR_VFLAG_SET | ATR_VFLAG_MODIFY;
	return rc;
}

/*
 * set_node_props - the information in the props attribute "*new" is used to
 *		    update the information in the props attribute, "*pattr".
 *		    the mode of the update is goverened by the argument "op"
 *		    (SET,INCR,DECR)
 */

int	set_node_props( pattr, new, op)
	attribute	*pattr;		/*attribute can get modified */
	attribute	*new;		/*carries new, modifying info*/
	enum batch_op	 op;
{
	struct prop	*pp, *xppx;
	struct prop	*newp, *oldp, *prev;
	int		 changed = 0;   /*if != 0 pattr has gotten changed*/
	int		 rc = 0;


	assert( pattr && new && (new->at_flags & ATR_VFLAG_SET) );
	pp = pattr->at_val.at_prop;
	xppx = new->at_val.at_prop;

	if( !xppx )
	    return (PBSE_INTERNAL);	/*VFLAG_SET on but no ptr to the data*/

	switch( op ) {

	    case SET: /*swap the the two prop lists*/
	    pattr->at_val.at_prop = xppx;
	    pattr->at_flags |= ATR_VFLAG_SET | ATR_VFLAG_MODIFY;
	    new->at_val.at_prop = pp;
	    break;

	    case INCR:     /*take props from "new" and place on front of pattr list*/
	    for( newp = xppx, prev = newp ; newp; /*newp,prev updating varies*/ ) {

		 for( oldp = pp; oldp; oldp = oldp->next ) {

		      if( !strcmp(newp->name, oldp->name) )
		          break;
		 }

		 if (!oldp) { /* name not in current list, add it to list*/

		     if( xppx == newp ) { /*go from front of "new" to front of current*/
			 xppx = newp->next;
			 prev = newp->next;
		         newp->next = pp;
		         pp = newp;
			 newp = prev;
		     }
		     else { /*go from interior of "new" to front of current*/
		         prev->next = newp->next;
		         newp->next = pp;
		         pp = newp;
			 newp = prev->next;
		     }

		     changed =1;
		 }
		 else { /*name in current prop list, don't add again*/

		     prev = newp;
		     newp = newp->next; 
		 }

	    } /*Bottom of outer "for"*/

	    if( changed ) {

		new->at_val.at_prop = xppx;
		pattr->at_val.at_prop = pp;
		pattr->at_flags |= ATR_VFLAG_SET | ATR_VFLAG_MODIFY;
	    }
	    break;
			    
	    case DECR:     /*take props from "pattr" and place on front of "new" list*/
	    for( newp = xppx ; newp; newp = newp->next ) {

		 for( oldp = pp, prev = oldp; oldp; oldp = oldp->next ) {

		      if( !strcmp(newp->name, oldp->name) )
			  break;
		      prev = oldp;
		 }

		 if ( oldp ) { /*found in current list, eliminate*/

		     if( pp == oldp ) { /*matched at the beginning*/
			 pp = oldp->next;

		     }
		     else { /*matched in the middle*/
			prev->next = oldp->next;
		     }

		     oldp->next = xppx;	/*add discarded prop to front of "new"*/
		     xppx = oldp;
		     changed =1;
		 }

	    } /*Bottom of outer "for"*/


	    if( changed ) {

		new->at_val.at_prop = xppx;
		pattr->at_val.at_prop = pp;
		pattr->at_flags |= ATR_VFLAG_SET | ATR_VFLAG_MODIFY;
	    }
	    break;

	    default:	rc = PBSE_INTERNAL;
	} /* Bottom of large switch */

	return rc;
}


/*
 * set_nodeflag - Use the input string's value to set a bit in
 *		  the "flags" variable pointed to by pflags
 *		  Each call sets one more bit in the flags
 *		  variable or it clears the flags variable
 *		  in the case where *str is the value "free"
 */

static int	set_nodeflag( str, pflag)
		char	*str;
		short	*pflag;
{
	int	rc = 0;

	if( *str == '\0' )
	    return (PBSE_BADNDATVAL);

	if( !strcmp(str, ND_free) )
	    *pflag = 0;
	else if( !strcmp(str, ND_offline) )
	    *pflag = *pflag | INUSE_OFFLINE ;
	else if( !strcmp(str, ND_down) )
	    *pflag = *pflag | INUSE_DOWN ;
	else if( !strcmp(str, ND_reserve) )
	    *pflag = *pflag | INUSE_RESERVE ;
	else {
	     rc = PBSE_BADNDATVAL;
	}
		
	return rc;
}


/*
 * node_state - Either derive a "state" attribute from the node
 *		or update node's "inuse" field using the "state"
 *		attribute.
 */

int	node_state( new, pnode, actmode)
	attribute	*new;		/*derive state into this attribute*/
	void		*pnode;		/*pointer to a pbsnode struct    */
	int		 actmode;	/*action mode; "NEW" or "ALTER"   */
{
	int rc = 0;
	struct pbsnode* np;


	np = (struct pbsnode*)pnode;	/*because of def of at_action  args*/
	switch( actmode ) {

	    case ATR_ACTION_NEW:  /*derive attribute*/
	    new->at_val.at_short = np->inuse;	
	    break;

	    case ATR_ACTION_ALTER:
	    np->inuse = new->at_val.at_short;
	    break;

	    default: rc = PBSE_INTERNAL;
	}
	return rc;
}


/*
 * node_ntype - Either derive an "ntype" attribute from the node
 *		or update node's "ntype" field using the 
 *		attribute's data
 */

int	node_ntype ( new, pnode, actmode)
	attribute	*new;		/*derive ntype into this attribute*/
	void		*pnode;		/*pointer to a pbsnode struct     */
	int		 actmode;	/*action mode; "NEW" or "ALTER"   */
{
	int rc = 0;
	struct pbsnode* np;


	np = (struct pbsnode*)pnode;	/*because of def of at_action  args*/
	switch( actmode ) {

	    case ATR_ACTION_NEW:  /*derive attribute*/
	    new->at_val.at_short = np->ntype;	
	    break;

	    case ATR_ACTION_ALTER:
	    np->ntype = new->at_val.at_short;
	    break;

	    default: rc = PBSE_INTERNAL;
	}
	return rc;
}

/*
 * node_prop_list - Either derive a "prop list" attribute from the node
 *		    or update node's prop list from attribute's prop list.
 */

int	node_prop_list( new, pnode, actmode)
	attribute	*new;		/*derive props into this attribute*/
	void		*pnode;		/*pointer to a pbsnode struct     */
	int		 actmode;	/*action mode; "NEW" or "ALTER"   */
{
	int		rc = 0, i;
	int		pcnt;		/*local prop counter*/
	struct prop	*tprop, *pp;	/*use as temporaries*/
	struct prop	*npp;
	struct prop	*prev;
	struct pbsnode* np;

	np = (struct pbsnode*)pnode;	/*because of at_action arg type*/
	switch( actmode ) {

	    case ATR_ACTION_NEW:	/*derive attribute; nprops counts name too */
					/*error handled by subsequent call to      */
					/*      attr_atomic_kill	           */

	    npp = np->first;
	    new->at_val.at_prop = (struct prop *)0;
	    prev = new->at_val.at_prop;
	    for(i=0; i<np->nprops -1; i++, npp = npp->next) {

		tprop = malloc(sizeof(struct prop));
		if( !tprop ) {

		    /*cleanup: caller invokes attr_atomic_kill if rc != 0   */
		    /*any hanging structures will be freed along with attrib*/
		    rc = PBSE_SYSTEM;
		    break;
		}
		tprop->next = (struct prop *)0;
		tprop->mark = npp->mark;
		if( prev ) {
		    prev->next = tprop;
		    prev = tprop;
		} else {
		    prev = tprop;
		    new->at_val.at_prop = tprop;
		}

		if( tprop->name = malloc( strlen(npp->name) + 1) )
		    strcpy(tprop->name, npp->name);

		else {

		    /*cleanup: caller invokes attr_atomic_kill if rc != 0   */
		    /*any hanging structures will be freed along with attrib*/
		    rc = PBSE_SYSTEM;
		    break;
		}

	    } /*bottom of the "for" loop*/
	    break;
				

	    case ATR_ACTION_ALTER:
	    tprop = (struct prop *)malloc(sizeof(struct prop));
	    if (!tprop) {
	        rc = PBSE_SYSTEM;
	        break;
	    }
	    tprop->next = (struct prop *)0;
	    tprop->name = malloc(strlen(np->last->name) + 1);
	    
	    if (!tprop->name ) {
		free( tprop );
	        rc = PBSE_SYSTEM;
	        break;
	    } 

	    strcpy(tprop->name, np->last->name);

	    pcnt = 0;
	    pp = new->at_val.at_prop;
	    while( pp ) {
	      pcnt++;
	      if( pp->next == 0 )
		  break;
	      else
		  pp = pp->next;
		    
	    }
		
	    if( pp )
	        pp->next = tprop;	      /*non-empty prop list; add node name prop*/
	    else
	        new->at_val.at_prop = tprop;  /*empty prop list; just the node name    */

	    np->nprops = pcnt + 1;	      /* +1 for the name  */
	    np->last  = tprop;
	    np->first = new->at_val.at_prop;
	    break;

	    default:	rc = PBSE_INTERNAL;
	}

	return rc;
}
