/*
This program is confidential and proprietary to MRJ Technology Solutions
and may not be reproduced, published or disclosed to others without
written authorization from MRJ.

Copyright (c) 1998 MRJ Technology Solutions, Inc. All Rights Reserved.

This program is derived from prior work some of which was originally
written as a joint project between the Numerical Aerospace Simulation
(NAS) Systems Division of NASA Ames Research Center and the National
Energy Research Supercomputer Center (NERSC) of Lawrence Livermore
National Laboratory.
*/

#include <pbs_config.h>   /* the master config generated by configure */

#include <ctype.h>
#include <stddef.h>
#include <string.h>

#include "Long.h"
#include "Long_.h"

#undef strTouL

#ifndef TRUE
#define TRUE 1
#define FALSE 0
#endif

static unsigned x_val;
static const char ident[] = "@(#) $RCSfile: strTouL.c,v $ $Revision: 2.0 $";
static const char letters[] = "abcdefghijklmnopqrstuvwxyz";
static char table[UCHAR_MAX + 1];

/*
 * strTouL - returns the unsigned Long value representing the string whose
 * 	     first character is *nptr, when interpreted as an integer in base,
 *	     base.
 *
 * 	If base is zero, the base of the integer is determined by the way the
 * 	string starts.  The string is interpreted as decimal if the first
 * 	character after leading white space and an optional sign is a digit
 * 	between 1 and 9, inclusive.  The string is interpreted as octal if the
 * 	first character after leading white space and an optional sign is the
 * 	digit "0" and the next character is not an "X" (either upper or lower
 * 	case).  The string is interpreted as hexidecimal if the first character
 * 	after leading white space and an optional sign is the digit "0",
 * 	followed by an "X" (either upper or lower case).
 *
 * 	If base is greater than 1 and less than the number of characters in the
 *	Long_dig array, it represents the base in which the number will be
 *	interpreted.  Characters for digits beyond 9 are represented by the
 *	letters of the alphabet, either upper case or lower case.
 */

u_Long strTouL(nptr, endptr, base)
    const char		*nptr;
    char		**endptr;
    int			base;
{
	unsigned	digit;
	u_Long		limit, value;
	enum {
	    unknown1,
	    unknown2,
	    hex1,
	    hex2,
	    hex3,
	    known,
	    working,
	    overflow
	}		state;

	if (table[(unsigned char)'1'] != 1) {
		int	i;		/* Initialize conversion table */

		(void)memset(table, CHAR_MAX, sizeof(table));
		for (i = (int)strlen(Long_dig) - 1; i >= 0; i--)
			table[(unsigned char)Long_dig[i]] = i;
		for (i = (int)strlen(letters) - 1; i >= 0; i--)
			table[(unsigned char)letters[i]] = i + 10;
		x_val = table[(unsigned char)'x'];
	}
	if (nptr == NULL) {
		if (endptr != NULL)
		        *endptr = (char *)nptr;
	        return (0);
	}
	if (base < 0 || base == 1 || (size_t)base > strlen(Long_dig)) {
	        errno = EDOM;
		if (endptr != NULL)
		        *endptr = (char *)nptr;
	        return (0);
	}
	switch (base) {
	    case 0:
		state = unknown1;
		break;
	    case 16:
		state = hex1;
		break;
	    default:
		state = known;
	}
	while (isspace(*nptr++));
	Long_neg = FALSE;
	switch (*--nptr) {
	    case '-':
		Long_neg = TRUE;
	    case '+':
		nptr++;
	}
        value = 0;
	while ((digit = table[(unsigned char)*nptr++]) != CHAR_MAX) {
		switch (state) {
		    case unknown1:
			if (digit >= 10)
				goto done;
			if (digit == 0) {
				state = unknown2;
				break;
			}
			base = 10;
			state = working;
			limit = UlONG_MAX / 10;
			value = digit;
			break;
		    case unknown2:
			if (digit >= 8) {
				if (digit != x_val)
					goto done;
				base = 16;
				state = hex3;
				break;
			}
			base = 8;
			state = working;
			limit = UlONG_MAX / 8;
			value = digit;
			break;
		    case hex1:
			if (digit >= base)
			        goto done;
			if (digit == 0) {
				state = hex2;
				break;
			}
			state = working;
			limit = UlONG_MAX / 16;
			value = digit;
			break;
		    case hex2:
			if (digit == x_val) {
				state = hex3;
				break;
			}
		    case hex3:
		    case known:
			if (digit >= base)
				goto done;
			state = working;
			limit = UlONG_MAX / base;
			value = digit;
			break;
		    case working:
			if (digit >= base)
			        goto done;
			if (value < limit) {
				value = value * base + digit;
				break;
			}
			if (value > limit ||
			    UlONG_MAX - (value *= base) < digit) {
				state = overflow;
				value = UlONG_MAX;
				errno = ERANGE;
				break;
			}
			value += digit;
			break;
		    case overflow:
			if (digit >= base)
			        goto done;
		}
	}
    done:
	if (endptr != NULL) {
		if (state == hex3)
		        nptr--;
	        *endptr = (char *)--nptr;
	}
        if (Long_neg)
                return (-value);
        return (value);
}
