/*
This program is confidential and proprietary to MRJ Technology Solutions
and may not be reproduced, published or disclosed to others without
written authorization from MRJ.

Copyright (c) 1998 MRJ Technology Solutions, Inc. All Rights Reserved.

This program is derived from prior work some of which was originally
written as a joint project between the Numerical Aerospace Simulation
(NAS) Systems Division of NASA Ames Research Center and the National
Energy Research Supercomputer Center (NERSC) of Lawrence Livermore
National Laboratory.
*/

#include <pbs_config.h>   /* the master config generated by configure */

#include <ctype.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "pbs_ifl.h"
#include "cmds.h"

static char ident[] = "@(#) $RCSfile: parse_depend.c,v $ $Revision: 2.0 $";

static char *deptypes[] = {
	"on",		/* "on" and "synccount" must be first two */
	"synccount",
	"after",
	"afterok",
	"afternotok",
	"afterany",
	"before",
	"beforeok",
	"beforenotok",
	"beforeany",
	"syncwith",
	(char *)0
};

/*
 *
 * parse_depend_item
 *
 * syntax:
 *
 *	jobid[:jobid...]
 *
 */

int parse_depend_item(depend_list, rtn_list, rtn_size)
	char *depend_list;
	char *rtn_list;		/* expanded jobids appended here */
	int   rtn_size;		/* size of above 		 */
{
    char *at;
    int i;
    int first = 1;
    char *s, *c;
    char full_job_id[PBS_MAXCLTJOBID+1];
    char server_out[PBS_MAXSERVERNAME + PBS_MAXPORTNUM + 2];

    /* Begin the parse */
    c = depend_list;

    /* Loop on strings between colons */
    while ( *c != '\0' ) {
	s = c;
        while (((*c != ':') || (*(c-1) == '\\'))  && *c ) c++;
	if ( s == c ) return 1;

	if ( *c == ':' ) {
		*c++ = '\0';
	}

	if (first) {
		first = 0;
		for (i=0; deptypes[i]; ++i) {
			if (strcmp(s, deptypes[i]) == 0)
				break;
		}
		if (deptypes[i] == (char *)0)
			return 1;
		(void)strcat(rtn_list, deptypes[i]);

	} else {

		if (i < 2) {		/* for "on" and "synccount", number */
			(void)strcat(rtn_list, s);
		} else {		/* for others, job id */
			at = strchr(s, (int)'@');
			if (get_server(s, full_job_id, server_out) != 0)
				return 1;
			(void)strcat(rtn_list, full_job_id);
			if (at) {
				(void)strcat(rtn_list, "@");
				(void)strcat(rtn_list, server_out);
			}
		}
	}
	if (*c)
		(void)strcat(rtn_list, ":");
    }
    if ( s == c ) return 1;

    return 0;
}



/*
 *
 * parse_depend_list
 *
 * syntax:
 *
 *	depend_list[,depend_list...]
 *
 * Arguments:
 *
 *	list		List of colon delimited lists.
 *
 */

int parse_depend_list(list, rtn_list, rtn_size)
	char *list;
	char *rtn_list;		/* expanded list returned here */
	int   rtn_size;		/* size of above array */

{
    char *b, *c, *s, *lc;
    int comma;

    if ( strlen(list) == 0 ) return (1);

    if ( (lc = (char *)malloc(strlen(list)+1)) == (char *)0 ) {
        fprintf(stderr, "Out of memory.\n");
        exit(1);
    }
    strcpy(lc, list);
    c = lc;
    *rtn_list = '\0';

    while ( *c != '\0' ) {
	/* Drop leading white space */
        while ( isspace(*c) ) c++;

	/* Find the next comma */
        s = c;
        while ( *c != ',' && *c ) c++;

	/* Drop any trailing blanks */
        comma = (*c == ',');
        *c = '\0';
	b = c - 1;
	while ( isspace((int)*b) ) *b-- = '\0';

	
	/* Parse the individual list item */

        if ( parse_depend_item(s, rtn_list, rtn_size) ) {
            return 1;
        }

        if ( comma ) {
            c++;
	    (void)strcat(rtn_list, ",");
        }
    }
    free(lc);

    if ( comma ) return 1;

    return 0;
}
