/*
This program is confidential and proprietary to MRJ Technology Solutions
and may not be reproduced, published or disclosed to others without
written authorization from MRJ.

Copyright (c) 1998 MRJ Technology Solutions, Inc. All Rights Reserved.

This program is derived from prior work some of which was originally
written as a joint project between the Numerical Aerospace Simulation
(NAS) Systems Division of NASA Ames Research Center and the National
Energy Research Supercomputer Center (NERSC) of Lawrence Livermore
National Laboratory.
*/
/*
 *  parse_jobid
 *
 * takes a job_id string as input, parses it to separate the 
 * 'current_server[:port]' part, and returns this in the return 
 * argument 'server'; if things go OK, the function value is
 * set to 0, if errors, it is set to 1.   The 'current_server[:port]'
 * part of 'job_id' is removed, if it was present.
 *
 * Full legal syntax is:
 *  seq_number[.parent_server[:port]][@current_server[:port]]  
 */

static char ident[] = "@(#) $RCSfile: parse_jobid.c,v $ $Revision: 2.0 $";

#include <pbs_config.h>   /* the master config generated by configure */

#include <string.h>
#include <ctype.h>
#include "pbs_ifl.h"

#define MAXSERVERNAME PBS_MAXSERVERNAME+PBS_MAXPORTNUM+2
#define ISNAMECHAR(x) ( (isgraph(x)) && ((x) != '#') && ( (x) != '@') )

static char seq_number[PBS_MAXSEQNUM+1]; 
static int s_pos;
static char parent_server[MAXSERVERNAME]; 
static int p_pos;
static char current_server[MAXSERVERNAME]; 
static int c_pos;

int parse_jobid(job_id, arg_seq_number, arg_parent_server, arg_current_server)
char *job_id;
char **arg_seq_number;
char **arg_parent_server;
char **arg_current_server;
{
    int i;
    char *c;

    /* initialize static data for this parsing call */
    for ( i=0; i<PBS_MAXSEQNUM+1; i++ ) seq_number[i]='\0';
    s_pos = 0;
    for ( i=0; i<MAXSERVERNAME; i++ ) parent_server[i]='\0';
    p_pos = 0;
    for ( i=0; i<MAXSERVERNAME; i++ ) current_server[i]='\0';
    c_pos = 0;

    /* Begin the parse */
    c = job_id;
    while ( isspace(*c) ) c++;

    /* Looking for a seq_number */
    while ( *c != '\0' ) {
        if ( isdigit(*c) ) {
	    if ( s_pos >= PBS_MAXSEQNUM ) return 3;
            seq_number[s_pos++]=*c;
        } else
            break;
        c++;
    }
    if ( s_pos == 0 ) return 1;

    /* Looking for a parent_server */
    if ( *c == '.' ) {
        c++;
        while ( *c != '\0' ) {
            if ( ISNAMECHAR(*c) ) {
	        if ( p_pos >= MAXSERVERNAME ) return 3;
                parent_server[p_pos++]=*c;
            } else
                break;
            c++;
        }
        if ( p_pos == 0 ) return 1;
    }

    /* Looking for a current_server */
    if ( *c == '@' ) {
        c++;
        while ( *c != '\0' ) {
            if ( ISNAMECHAR(*c) ) {
	        if ( c_pos >= MAXSERVERNAME ) return 3;
                current_server[c_pos++]=*c;
            } else
                break;
            c++;
        }
        if ( c_pos == 0 ) return 1;
    }

    if ( *c != '\0' ) return 2;

    /* set char * pointers to static data, to arguments */
    if ( arg_seq_number != NULL ) *arg_seq_number = seq_number;
    if ( arg_parent_server != NULL ) *arg_parent_server = parent_server;
    if ( arg_current_server != NULL ) *arg_current_server = current_server;

    return 0;
}
