/*
This program is confidential and proprietary to MRJ Technology Solutions
and may not be reproduced, published or disclosed to others without
written authorization from MRJ.

Copyright (c) 1998 MRJ Technology Solutions, Inc. All Rights Reserved.

This program is derived from prior work some of which was originally
written as a joint project between the Numerical Aerospace Simulation
(NAS) Systems Division of NASA Ames Research Center and the National
Energy Research Supercomputer Center (NERSC) of Lawrence Livermore
National Laboratory.
*/

#include <pbs_config.h>   /* the master config generated by configure */

#include <ctype.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "pbs_ifl.h"
#ifndef NULL
#define NULL 0
#endif

static char ident[] = "@(#) $RCSfile: parse_stage.c,v $ $Revision: 2.0 $";

#define ISNAMECHAR(x) ( (isgraph(x)) && ((x) != '@') && ((x) != ':') )

/*
 *
 * parse_stage_name
 *
 * syntax:
 *
 *	locat_file@hostname:remote_file
 *
 * Arguments:
 *
 *	pair	A staged file name pair.
 *	local	Local file name.
 *	host	Remote host.
 *	remote	Remote file name.
 *
 */

int
parse_stage_name(pair, local, host, remote)
char *pair;
char **local;
char **host;
char **remote;
{
    int i;
    char *c;
    static char local_name[MAXPATHLEN+1];
    static int l_pos;
    static char host_name[PBS_MAXSERVERNAME];
    static int h_pos;
    static char remote_name[MAXPATHLEN+1];
    static int r_pos;

    /* initialize static data for this parsing call */
    for ( i=0; i<MAXPATHLEN+1; i++ ) local_name[i]='\0';
    l_pos= 0;
    for ( i=0; i<PBS_MAXSERVERNAME; i++ ) host_name[i]='\0';
    h_pos = 0;
    for ( i=0; i<MAXPATHLEN+1; i++ ) remote_name[i]='\0';
    r_pos= 0;

    /* Begin the parse */
    c = pair;
    while ( isspace(*c) ) c++;

    /* Looking for something before the @ sign */
    while ( *c != '\0' ) {
        if ( ISNAMECHAR(*c) ) {
            if ( l_pos >= MAXPATHLEN ) return 1;
            local_name[l_pos++]=*c;
        } else
            break;
        c++;
    }
    if ( l_pos == 0 ) return 1;

    /* Looking for something between the @ and the : */
    if ( *c == '@' ) {
        c++;
        while ( *c != '\0' ) {
            if ( ISNAMECHAR(*c) ) {
                if ( h_pos >= PBS_MAXSERVERNAME ) return 1;
                host_name[h_pos++]=*c;
            } else
                break;
            c++;
        }
    }
    if ( h_pos == 0 ) return 1;

    /* Looking for something after the : */
    if ( *c == ':' ) {
        c++;
        while ( *c != '\0' ) {
            if ( ISNAMECHAR(*c) ) {
                if ( r_pos >= MAXPATHLEN ) return 1;
                remote_name[r_pos++]=*c;
            } else
                break;
            c++;
        }
    }
    if ( r_pos == 0 ) return 1;

    if ( *c != '\0' ) return 1;

    /* set char * pointers to static data, to arguments */
    if ( local  != NULL ) *local = local_name;
    if ( host   != NULL ) *host = host_name;
    if ( remote != NULL ) *remote = remote_name;

    return (0);
}



/*
 *
 * parse_stage_list
 *
 * syntax:
 *
 *	local_file@hostname:remote_file [,...]
 *
 * Arguments:
 *
 *	list	List of staged file name pairs.
 *
 */

int
parse_stage_list(list)
char *list;
{
    char *b, *c, *s, *l;
    int comma;
    char *local, *host, *remote;

    if ( strlen(list) == 0 ) return (1);

    if ( (l = (char *)malloc(strlen(list)+1)) == (char *)0 ) {
        fprintf(stderr, "Out of memory.\n");
        exit(1);
    }
    strcpy(l, list);
    c = l;
    while ( *c != '\0' ) {
	/* Drop leading white space */
	while ( isspace((int)*c) ) c++;

	/* Find the next comma */
        s = c;
        while ( *c != '\0' && *c != ',' ) c++;

	/* Drop any trailing blanks */
        comma = (*c == ',');
        *c = '\0';
	b = c - 1;
	while ( isspace((int)*b) ) *b-- = '\0';

	/* Parse the individual list item */
        if ( parse_stage_name(s, &local, &host, &remote) ) {
	    (void)free(l);
            return 1;
        }

	/* Make sure all parts of the item are present */
        if ( strlen(local)  == 0 ) {
		(void)free(l);
		return 1;
	}
        if ( strlen(host)   == 0 ) {
		(void)free(l);
		return 1;
	}
        if ( strlen(remote) == 0 ) {
		(void)free(l);
		return 1;
	}

        if ( comma ) {
            c++;
        }
    }
    if ( comma ) {
		(void)free(l);
		return 1;
	}

    (void)free(l);

    return 0;
}
