/*
This program is confidential and proprietary to MRJ Technology Solutions
and may not be reproduced, published or disclosed to others without
written authorization from MRJ.

Copyright (c) 1998 MRJ Technology Solutions, Inc. All Rights Reserved.

This program is derived from prior work some of which was originally
written as a joint project between the Numerical Aerospace Simulation
(NAS) Systems Division of NASA Ames Research Center and the National
Energy Research Supercomputer Center (NERSC) of Lawrence Livermore
National Laboratory.
*/
/*
 * perpare_path
 *
 *	Perpare a full path name to give to the server.
 *
 */

#include <pbs_config.h>   /* the master config generated by configure */

#include <ctype.h>
#include <errno.h>
#include <netdb.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/stat.h>
#include "pbs_ifl.h"
#include "net_connect.h"

static char ident[] = "@(#) $RCSfile: prepare_path.c,v $ $Revision: 2.0 $";

int 
prepare_path(path_in, path_out)
char *path_in;
char *path_out;
{
    int i;
    char *c;
    char host_name[PBS_MAXSERVERNAME+1];
    int h_pos;
    char path_name[MAXPATHLEN+1];
    int p_pos;
    char cwd[MAXPATHLEN+1];
    char *host_given;
    struct	stat	statbuf;
    dev_t	dev;
    ino_t	ino;

/* initialize data for this parsing call */
    for ( i=0; i<= PBS_MAXSERVERNAME; i++ ) host_name[i]='\0';
    h_pos = 0;
    for ( i=0; i<= MAXPATHLEN; i++ ) path_name[i]='\0';
    p_pos = 0;
    cwd[MAXPATHLEN] = '\0';

/* Begin the parse */
    c = path_in;
    while ((int)isspace(*c)) c++;
    if ( strlen(c) == 0 ) return 1;

/* Looking for a hostname :  */
    if ( (host_given=strchr(path_in, ':')) != NULL ) {
        while ( *c != ':' && *c != '\0' ) {
            if ( isalnum(*c) || (*c == '.') || (*c == '-') || (*c == '_') )
                host_name[h_pos++]=*c;
            else
                break;
            c++;
        }
    }

/* Looking for a posix path */
    if ( (*c == ':') || (c == path_in) ) {
        if ( *c == ':' ) c++;
        while ( *c != '\0' ) {
            if ( isgraph(*c) )
                path_name[p_pos++]=*c;
            else
                break;
            c++;
        }
    }

    /* we had trailing trash, or a parse error */
    if ( *c != '\0' ) return 1;
    if ( strlen(path_name) == 0 ) return 1;

/* get full host name */
    if ( host_name[0] == '\0' ) {
        if ( gethostname(host_name, PBS_MAXSERVERNAME) != 0 ) return 2;
    }
    if ( get_fullhostname(host_name, host_name, PBS_MAXSERVERNAME) != 0 ) return 2;

/* prepare complete path name */
    strcpy(path_out, host_name);
    strcat(path_out, ":");
    if ( path_name[0] != '/' && host_given == NULL ) {
     	c = getenv("PWD");		/* PWD carries a name that will cause */
 	if ( c != NULL ) {		/* the NFS to mount */
 	
 	    if (stat(c, &statbuf) < 0) {	/* can't stat PWD */
 		c = NULL;
     	    } else {
 		dev = statbuf.st_dev;
 		ino = statbuf.st_ino;	
 		if (stat(".", &statbuf) < 0) {
 		    perror("prepare_path: cannot stat current directory:" );
 		    return (1);
     		}
 	    }
 	    if (dev == statbuf.st_dev && ino == statbuf.st_ino) {
         	strcpy(cwd, c);
 	    } else {
 		c = NULL;
 	    }
 	}
 	if (c == NULL) {
 	    c = getcwd(cwd, MAXPATHLEN);
 	    if ( c == NULL ) {
 		perror("prepare_path: getcwd failed : ");
 		return (1);
 	    }
 	}
        strcat(path_out, cwd);
        strcat(path_out, "/");
    }
    strcat(path_out, path_name);

    return (0);
}
