/*
This program is confidential and proprietary to MRJ Technology Solutions
and may not be reproduced, published or disclosed to others without
written authorization from MRJ.

Copyright (c) 1998 MRJ Technology Solutions, Inc. All Rights Reserved.

This program is derived from prior work some of which was originally
written as a joint project between the Numerical Aerospace Simulation
(NAS) Systems Division of NASA Ames Research Center and the National
Energy Research Supercomputer Center (NERSC) of Lawrence Livermore
National Laboratory.
*/
/*
 * set_resources
 *
 *	Append a entries to the attribute list that are from the resource list.
 * If the add flag is set, append the resource regardless. Otherwise, append
 * it only if it is not already on the list.
 *
 */

#include <pbs_config.h>   /* the master config generated by configure */

#define TRUE	1
#define FALSE	0

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include "pbs_ifl.h"

static char ident[] = "@(#) $RCSfile: set_resource.c,v $ $Revision: 2.0 $";

int
set_resources(attrib, resources, add)
struct attrl **attrib;
char *resources;
int add;
{
    char *r, *eq, *v, *e;
    char *str;
    struct attrl *attr, *ap;
    int found;

    r = resources;
    while ( *r != '\0' ) {

	/* Get the resource name */
        eq = r;
	while ( *eq != '=' && *eq != ',' && *eq != '\0' ) eq++;
	
	/* Make sure there is a resource name */
        if ( r == eq ) {
            return (1);
        }

	/* If separated by an equal sign, get the value */
	if ( *eq == '=' ) {
	    v = eq + 1;
	    while ( isspace((int)*v) ) v++;
	    e = v;
	    while ( *e != ',' && *e != '\0' ) {
		if ( isspace((int)*e) ) return(1);
	        e++;
	    }
	} else
	    v = NULL;
	
        /* Allocate memory for the attrl structure */
        attr = (struct attrl *) malloc(sizeof(struct attrl));
        if ( attr == NULL ) {
            fprintf(stderr, "Out of memory\n");
            exit(2);
        }
	
        /* Allocate memory for the attribute name and copy */
        str = (char *) malloc( strlen(ATTR_l)+1 );
        if ( str == NULL ) {
            fprintf(stderr, "Out of memory\n");
            exit(2);
        }
        strcpy(str, ATTR_l);
        attr->name = str;
	
        /* Allocate memory for the resource name and copy */
        str = (char *) malloc( eq-r+1 );
        if ( str == NULL ) {
            fprintf(stderr, "Out of memory\n");
            exit(2);
        }
        strncpy(str, r, eq-r);
        str[eq-r] = '\0';
        attr->resource = str;
	
        /* Allocate memory for the value and copy */
	if ( v != NULL ) {
            str = (char *) malloc(e-v+1);
            if ( str == NULL ) {
                fprintf(stderr, "Out of memory\n");
                exit(2);
            }
            strncpy(str, v, e-v);
            str[e-v] = '\0';
            attr->value = str;
	} else {
	    str = (char *) malloc(1);
	    if ( str == NULL ) {
                fprintf(stderr, "Out of memory\n");
                exit(2);
            }
	    str[0] = '\0';
	    attr->value = str;
	}

        /* Put it on the attribute list */
	/* If the argument add is true, add to the list regardless.
	 * Otherwise, add it to the list only if the resource name
	 * is not already on the list.
	 */
        attr->next = NULL;
        if ( *attrib == NULL ) {
            *attrib = attr;
        } else {
            ap = *attrib;
	    found = FALSE;
	    while ( ap->next != NULL ) {
	        if ( strcmp(ap->name, ATTR_l) == 0 &&
		     strcmp(ap->resource, attr->resource) == 0 ) found = TRUE;
	        ap = ap->next;
	    }
	    if ( add || !found ) ap->next = attr;
        }

	/* Get ready for next resource/value pair */
	if ( v != NULL )
	    r = e;
	else
            r = eq;
	if ( *r == ',' ) {
	    r++;
	    if ( *r == '\0' ) {
                return (1);
	    }
	}
    }
    return (0);
}
