#include <pbs_config.h>   /* the master config generated by configure */

#include <assert.h>
#include <float.h>
#include <stddef.h>
#include <stdlib.h>

#include <dis.h>

unsigned dis_dmx10 = 0;
double *dis_dp10 = NULL;
double *dis_dn10 = NULL;

/*
 *	Allocate and fill tables with all powers of 10 that fit the forms:
 *
 *				  n
 *				 2
 *		*dis_dp10[n] = 10
 *
 *				  ( n)
 *				 -(2 )
 *		*dis_dn10[n] = 10
 *
 *	For all values of n supported by the floating point format.  Set
 *	dis_dmx10 equal to the largest value of n that fits the format.
 */

void disi10d_() {
	int		i;
	unsigned long	ul;
	dis_long_double_t	accum;
	size_t		tabsize;

	assert(dis_dp10 == NULL);
	assert(dis_dn10 == NULL);
	assert(dis_dmx10 == 0);

#if DBL_MAX_10_EXP + DBL_MIN_10_EXP > 0
	ul = DBL_MAX_10_EXP;
#else
	ul = -DBL_MIN_10_EXP;
#endif
	while (ul >>= 1)
		dis_dmx10++;
	tabsize = (dis_dmx10 + 1) * sizeof(double);
	dis_dp10 = (double *)malloc(tabsize);
	assert(dis_dp10 != NULL);
	dis_dn10 = (double *)malloc(tabsize);
	assert(dis_dn10 != NULL);
	assert(dis_dmx10 > 0);
	dis_dp10[0] = accum = 10.0L;
	dis_dn10[0] = 1.0L / accum;
	for (i = 1; i <= dis_dmx10; i++) {
		accum *= accum;
		dis_dp10[i] = accum;
		dis_dn10[i] = 1.0L / accum;
	}
}
