#include <pbs_config.h>   /* the master config generated by configure */

#include <assert.h>
#include <float.h>
#include <stddef.h>
#include <stdlib.h>

#include <dis.h>

unsigned dis_lmx10 = 0;
dis_long_double_t *dis_lp10 = NULL;
dis_long_double_t *dis_ln10 = NULL;

/*
 *	Allocate and fill tables with all powers of 10 that fit the forms:
 *
 *				  n
 *				 2
 *		*dis_lp10[n] = 10
 *
 *				  ( n)
 *				 -(2 )
 *		*dis_ln10[n] = 10
 *
 *	For all values of n supported by the floating point format.  Set
 *	dis_lmx10 equal to the largest value of n that fits the format.
 */

void disi10l_() {
	int		i;
	unsigned long	ul;
	dis_long_double_t	accum;
	size_t		tabsize;

	assert(dis_lp10 == NULL);
	assert(dis_ln10 == NULL);
	assert(dis_lmx10 == 0);

#if LDBL_MAX_10_EXP + LDBL_MIN_10_EXP > 0
	ul = LDBL_MAX_10_EXP;
#else
	ul = -LDBL_MIN_10_EXP;
#endif
	while (ul >>= 1)
		dis_lmx10++;
	tabsize = (dis_lmx10 + 1) * sizeof(dis_long_double_t);
	dis_lp10 = (dis_long_double_t *)malloc(tabsize);
	assert(dis_lp10 != NULL);
	dis_ln10 = (dis_long_double_t *)malloc(tabsize);
	assert(dis_ln10 != NULL);
	assert(dis_lmx10 > 0);
	dis_lp10[0] = accum = 10.0L;
	dis_ln10[0] = 1.0L / accum;
	for (i = 1; i <= dis_lmx10; i++) {
		accum *= accum;
		dis_lp10[i] = accum;
		dis_ln10[i] = 1.0L / accum;
	}
}
