/*
 * Synopsis:
 *	char *disrst(int stream, int *retval)
 *
 *	Gets a Data-is-Strings character string from <stream> and converts it
 *	into a null-terminated string, and returns a pointer to the result.  The
 *	character string in <stream> consists of an unsigned integer, followed
 *	by a number of characters determined by the unsigned integer.
 *
 *	*<retval> gets DIS_SUCCESS if everything works well.  It gets an error
 *	code otherwise.  In case of an error, the <stream> character pointer is
 *	reset, making it possible to retry with some other conversion strategy.
 *	In case of an error, the return value is NULL.
 */

#include <pbs_config.h>   /* the master config generated by configure */

#include <assert.h>
#include <stddef.h>
#include <stdlib.h>

#ifndef NDEBUG
#include <string.h>
#endif

#include "dis.h"
#include "dis_.h"

char *disrst(stream, retval)
    int			stream;
    int			*retval;
    {
	int		locret;
	int		negate;
	unsigned	count;
	char		*value = NULL;

	assert(retval != NULL);
	assert(dis_gets != NULL);
	assert(disr_commit != NULL);

	locret = disrsi_(stream, &negate, &count, 1);
	if (locret == DIS_SUCCESS) {
		if (negate)
			locret = DIS_BADSIGN;
		else {
			value = (char *)malloc((size_t)count+1);
			if (value == NULL)
		        	locret = DIS_NOMALLOC;
			else {
				if ((*dis_gets)(stream, value,
						(size_t)count) != (size_t)count)
			        	locret = DIS_PROTO;
#ifndef NDEBUG
				else if (memchr(value, 0, (size_t)count))
			        	locret = DIS_NULLSTR;
#endif
                        	else
			        	value[count] = '\0';
                	}
		}
	}
	locret = ((*disr_commit)(stream, locret == DIS_SUCCESS) < 0) ?
	    DIS_NOCOMMIT : locret;
	if ((*retval = locret) != DIS_SUCCESS && value != NULL) {
		free(value);
		value = NULL;
	}
        return (value);
}
