/*
 * Synopsis:
 *	int diswcs(int stream, char *value, size_t nchars)
 *
 *	Converts a counted string in *<value> into a Data-is-Strings character
 *	string and sends it to <stream>.  The character string in <stream>
 *	consists of the unsigned integer representation of <nchars>, followed by
 *	<nchars> characters from *<value>.
 *
 *	Returns DIS_SUCCESS if everything works well.  Returns an error code
 *	otherwise.  In case of an error, no characters are sent to <stream>.
 */

#include <pbs_config.h>   /* the master config generated by configure */

#include <assert.h>
#include <stddef.h>
#include <stdio.h>

#include "dis.h"
#include "dis_.h"

int diswcs(stream, value, nchars)
    int			stream;
    const char		*value;
    size_t		nchars;
    {
	int		retval;

	assert(disw_commit != NULL);
	assert(dis_puts != NULL);
	assert(nchars <= UINT_MAX);

	retval = diswui_(stream, (unsigned)nchars);
	if (retval == DIS_SUCCESS && nchars > 0 &&
	    (*dis_puts)(stream, value, nchars) != nchars)
	        retval = DIS_PROTO;
        return (((*disw_commit)(stream, retval == DIS_SUCCESS) < 0) ?
	       DIS_NOCOMMIT : retval);
}
