/*
 * Synopsis:
 * 	int diswsi(int stream, int value)
 *
 *	Converts <value> into a Data-is-Strings signed integer and sends it to
 *	<stream>.
 *
 *	This format for character strings representing integers can best be
 *	understood through the decoding algorithm:
 *
 *	1. Initialize the digit count to 1.
 *
 *	2. Read the next character; if it is a sign, go to step (4).
 *
 *	3. Decode a new count from the digit decoded in step (2) and the next
 *	   count - 1 digits; repeat step (2).
 *
 *	4. Decode the next count digits as the unsigned integer.
 *
 *	Returns DIS_SUCCESS if everything works well.  Returns an error code
 *	otherwise.  In case of an error, no characters are sent to <stream>.
 */

#include <pbs_config.h>   /* the master config generated by configure */

#include <assert.h>
#include <stddef.h>
#include <stdio.h>

#include "dis.h"
#include "dis_.h"
#undef diswsi

int diswsi(stream, value)
    int			stream;
    int			value;
    {
	int		retval;
	unsigned	ndigs;
	unsigned	uval;
	char		c;
	char		*cp;

	assert(stream >= 0);
	assert(dis_puts != NULL);
	assert(disw_commit != NULL);

	if (value < 0) {
		uval = (unsigned)-(value + 1) + 1;
		c = '-';
	} else {
		uval = value;
	        c = '+';
	}
	cp = discui_(&dis_buffer[DIS_BUFSIZ], uval, &ndigs);
	*--cp = c;
	while (ndigs > 1)
		cp = discui_(cp, ndigs, &ndigs);
	retval = (*dis_puts)(stream, cp,
			(size_t)(&dis_buffer[DIS_BUFSIZ] - cp)) < 0 ?
			DIS_PROTO : DIS_SUCCESS;
	return (((*disw_commit)(stream, retval == DIS_SUCCESS) < 0) ?
	       DIS_NOCOMMIT : retval);
}

