/*
This program is confidential and proprietary to MRJ Technology Solutions
and may not be reproduced, published or disclosed to others without
written authorization from MRJ.

Copyright (c) 1998 MRJ Technology Solutions, Inc. All Rights Reserved.

This program is derived from prior work some of which was originally
written as a joint project between the Numerical Aerospace Simulation
(NAS) Systems Division of NASA Ames Research Center and the National
Energy Research Supercomputer Center (NERSC) of Lawrence Livermore
National Laboratory.
*/
/*	PBS_rdrpy

	Read the reply to a batch request.
	A reply structure is allocated and cleared.
	The reply is read and decoded into the structure.
	The reply structure is returned.

	The caller MUST free the reply structure by calling
	PBS_FreeReply().
*/

#include <pbs_config.h>   /* the master config generated by configure */

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "libpbs.h"
#include "dis.h"

static char ident[] = "@(#) $RCSfile: PBSD_rdrpy.c,v $ $Revision: 2.0 $";

struct batch_reply *PBSD_rdrpy(c)
	int c;
{
	int		    rc;
	struct batch_reply *reply;
	int		    sock;

	/* clear any prior error message */

	if( connection[c].ch_errtxt != (char *)NULL ) {
		free( connection[c].ch_errtxt );
		connection[c].ch_errtxt = (char *)NULL;
	}

	if ((reply = (struct batch_reply *)malloc(sizeof (struct batch_reply))) == 0) {
		connection[c].ch_errno = PBSE_SYSTEM;
		pbs_errno = PBSE_SYSTEM;
		return ((struct batch_reply *)0);
	}
	(void)memset(reply, 0, sizeof (struct batch_reply));

	sock = connection[c].ch_socket;
	DIS_tcp_setup(sock);

	if (rc = decode_DIS_replyCmd(sock, reply))   {
	    (void)free(reply);
	    pbs_errno = PBSE_PROTOCOL;
	    connection[c].ch_errno = PBSE_PROTOCOL;
	    connection[c].ch_errtxt = strdup(dis_emsg[rc]);
	    return (struct batch_reply *)NULL;
	}
	DIS_tcp_reset(sock, 0);		/* reset DIS read buffer */

	connection[c].ch_errno = reply->brp_code;
	pbs_errno = reply->brp_code;
	if (reply->brp_choice == BATCH_REPLY_CHOICE_Text) {
		connection[c].ch_errtxt = strdup(reply->brp_un.brp_txt.brp_str);
	}
	return reply;
}


/*
 * PBS_FreeReply - Free a batch_reply structure allocated in PBS_rdrpy() 
 *
 *	Any additional allocated substructures pointed to from the
 *	reply structure are freed, then the base struture itself is gone.
 */

void PBSD_FreeReply(reply)
	struct batch_reply *reply;
{
	struct brp_select   *psel;
	struct brp_select   *pselx;
	struct brp_cmdstat  *pstc;
	struct brp_cmdstat  *pstcx;
	struct attrl        *pattrl;
	struct attrl	    *pattrx;

	if (reply == 0)
		return;
	if (reply->brp_choice == BATCH_REPLY_CHOICE_Text) {
		if (reply->brp_un.brp_txt.brp_str) {
			(void)free(reply->brp_un.brp_txt.brp_str);
			reply->brp_un.brp_txt.brp_str = (char *)0;
			reply->brp_un.brp_txt.brp_txtlen = 0;
		}

	} else if (reply->brp_choice == BATCH_REPLY_CHOICE_Select) {
		psel = reply->brp_un.brp_select;
		while (psel) {
		    pselx = psel->brp_next;
		    (void)free(psel);
		    psel = pselx;
		}
		
	} else if (reply->brp_choice == BATCH_REPLY_CHOICE_Status) {
		pstc = reply->brp_un.brp_statc;
		while (pstc) {
			pstcx = pstc->brp_stlink;
			pattrl = pstc->brp_attrl;
			while (pattrl) {
				pattrx = pattrl->next;
				if (pattrl->name)
					(void)free(pattrl->name);
				if (pattrl->resource)
					(void)free(pattrl->resource);
				if (pattrl->value)
					(void)free(pattrl->value);
				(void)free(pattrl);
				pattrl = pattrx;
			}
			(void)free(pstc);
			pstc = pstcx;
		}
	} else if (reply->brp_choice == BATCH_REPLY_CHOICE_RescQuery) {
		(void)free(reply->brp_un.brp_rescq.brq_avail);
		(void)free(reply->brp_un.brp_rescq.brq_alloc);
		(void)free(reply->brp_un.brp_rescq.brq_resvd);
		(void)free(reply->brp_un.brp_rescq.brq_down);
	}

	(void)free(reply);
}
