/*
This program is confidential and proprietary to MRJ Technology Solutions
and may not be reproduced, published or disclosed to others without
written authorization from MRJ.

Copyright (c) 1998 MRJ Technology Solutions, Inc. All Rights Reserved.

This program is derived from prior work some of which was originally
written as a joint project between the Numerical Aerospace Simulation
(NAS) Systems Division of NASA Ames Research Center and the National
Energy Research Supercomputer Center (NERSC) of Lawrence Livermore
National Laboratory.
*/
#include <pbs_config.h>   /* the master config generated by configure */

#include <string.h>
#include <unistd.h>
#include <stdio.h>
#include <fcntl.h>
#include "portability.h"
#include "libpbs.h"
#include "dis.h"

/*	PBS_rdytocmt.c

	This function does the Ready To Commit sub-function of
	the Queue Job request.
*/

static char ident[] = "@(#) $RCSfile: PBSD_submit.c,v $ $Revision: 2.0 $";

int PBSD_rdytocmt(connect, jobid)
	int connect;
	char * jobid;
{
	int	rc;
	struct batch_reply *reply;
	int	sock;

	sock = connection[connect].ch_socket;
	DIS_tcp_setup(sock);

	if ( (rc=encode_DIS_ReqHdr(sock, PBS_BATCH_RdytoCommit,pbs_current_user)) ||
	     (rc = encode_DIS_JobId(sock, jobid))  ||
	     (rc = encode_DIS_ReqExtend(sock, (char *)0)) ) {
		connection[connect].ch_errtxt = strdup(dis_emsg[rc]);
		return (pbs_errno = PBSE_PROTOCOL);
	}
	if (DIS_tcp_wflush(sock)) {
		return (pbs_errno = PBSE_PROTOCOL);
	}
	
	/* read reply */

	reply = PBSD_rdrpy(connect);

	PBSD_FreeReply(reply);

	return connection[connect].ch_errno;
}

/*	PBS_commit.c

	This function does the Commit sub-function of
	the Queue Job request.
*/

int PBSD_commit(connect, jobid)
	int connect;
	char * jobid;
{
	struct batch_reply *reply;
	int rc;
	int sock;
	
	sock = connection[connect].ch_socket;
	DIS_tcp_setup(sock);

	if ( (rc = encode_DIS_ReqHdr(sock, PBS_BATCH_Commit, pbs_current_user)) ||
	     (rc = encode_DIS_JobId(sock, jobid)) ||
	     (rc = encode_DIS_ReqExtend(sock, (char *)0)) ) {
		connection[connect].ch_errtxt = strdup(dis_emsg[rc]);
		return (pbs_errno = PBSE_PROTOCOL);
	}
	
	if (DIS_tcp_wflush(sock)) {
		return (pbs_errno = PBSE_PROTOCOL);
	}

	reply = PBSD_rdrpy(connect);

	PBSD_FreeReply(reply);

	return connection[connect].ch_errno;
}

/*	PBS_scbuf.c
 *
 *	Send a chunk of a of the job script to the server.
 *	Called by pbs_submit.  The buffer length could be
 *	zero; the server should handle that case...
*/

static int PBSD_scbuf(c, reqtype, seq, buf, len, jobid, which)
	int   c;		/* connection handle     */
	int   reqtype;		/* request type */
	int   seq;		/* file chunk sequence number */
	char *buf;		/* file chunk */
	int   len;		/* length of chunk */
	char *jobid;		/* job id (for types 1 and 2 only) */
	enum job_file which;	/* standard file type, see libpbs.h */
{
	struct batch_reply   *reply;
	int	rc;
	int	sock;

	sock = connection[c].ch_socket;
	DIS_tcp_setup(sock);

	if (jobid == (char *)0)
		jobid = "";	/* use null string for null pointer */

	if ( (rc = encode_DIS_ReqHdr(sock, reqtype, pbs_current_user)) ||
	     (rc = encode_DIS_JobFile(sock, seq, buf, len, jobid, which)) ||
	     (rc = encode_DIS_ReqExtend(sock, (char *)0)) ) {
		connection[c].ch_errtxt = strdup(dis_emsg[rc]);
		return (pbs_errno = PBSE_PROTOCOL);
	}
	if (DIS_tcp_wflush(sock)) {
		return (pbs_errno = PBSE_PROTOCOL);
	}
				
	/* read reply */

	reply = PBSD_rdrpy(c);
	
	PBSD_FreeReply(reply);

	return connection[c].ch_errno;
}

/*	PBS_jscript.c
 *
 *	The Job Script subfunction of the Queue Job request
 *	-- the function PBS_scbuf is called repeatedly to
 *	transfer chunks of the script to the server.
*/

int PBSD_jscript(c, script_file)
	int c;
	char *script_file;
{
	int i;
	int fd;
	int cc;
	char s_buf[SCRIPT_CHUNK_Z];
	
	if( (fd = open(script_file, O_RDONLY, 0)) < 0 ) {
		return (-1);
	}
	i = 0;
	cc = read(fd, s_buf, SCRIPT_CHUNK_Z);
	while ((cc > 0) && (PBSD_scbuf(c,PBS_BATCH_jobscript,i,s_buf,cc,(char *)0,JScript) == 0)) {
		i++;
		cc = read(fd, s_buf, SCRIPT_CHUNK_Z);
	}

	close(fd);
	if (cc < 0)	/* read failed */
		return (-1);
	return connection[c].ch_errno;
}

/*	PBS_jobfile.c
 *
 *	The Job File function used to move files related to
 *	a job between servers.
 *	-- the function PBS_scbuf is called repeatedly to
 *	transfer chunks of the script to the server.
*/

int PBSD_jobfile(c, req_type, path, jobid, which)
	int   c;
	int   req_type;
	char *path;
	char *jobid;
	enum job_file which;
{
	int   i;
	int   cc;
	int   fd;
	char  s_buf[SCRIPT_CHUNK_Z];
	
	if( (fd = open(path, O_RDONLY, 0)) < 0 ) {
		return (-1);
	}
	i = 0;
	cc = read(fd, s_buf, SCRIPT_CHUNK_Z);
	while ((cc > 0) && 
	       (PBSD_scbuf(c, req_type, i, s_buf, cc, jobid, which ) == 0) ) {
		i++;
		cc = read(fd, s_buf, SCRIPT_CHUNK_Z);
	}

	close(fd);
	if (cc < 0)	/* read failed */
		return (-1);
	return connection[c].ch_errno;
}

/*	PBS_queuejob.c

	This function sends the first part of the Queue Job request
*/

char * PBSD_queuejob(connect, jobid, destin, attrib, extend)
	int connect;
	char * jobid;
	char * destin;
	struct attropl * attrib;
	char * extend;
{
	struct batch_reply *reply;
	char  *return_jobid = (char *)NULL;
	int    rc;
	int    sock;
	
	sock = connection[connect].ch_socket;
	DIS_tcp_setup(sock);

	/* first, set up the body of the Queue Job request */
	
	if ( (rc = encode_DIS_ReqHdr(sock, PBS_BATCH_QueueJob, pbs_current_user)) ||
	     (rc = encode_DIS_QueueJob(sock, jobid, destin, attrib)) ||
	     (rc = encode_DIS_ReqExtend(sock, extend)) ) {
		connection[connect].ch_errtxt = strdup(dis_emsg[rc]);
		pbs_errno = PBSE_PROTOCOL;
		return return_jobid;
	}
	if (DIS_tcp_wflush(sock)) {
		pbs_errno = PBSE_PROTOCOL;
		return return_jobid;
	}

	/* read reply from stream into presentation element */

	reply = PBSD_rdrpy(connect);
	if ( reply == NULL ) {
                pbs_errno = PBSE_PROTOCOL;
	} else if ( reply->brp_choice &&
	            reply->brp_choice != BATCH_REPLY_CHOICE_Text &&
	            reply->brp_choice != BATCH_REPLY_CHOICE_Queue) {
		pbs_errno = PBSE_PROTOCOL;
	} else if( connection[connect].ch_errno == 0 )
		return_jobid = strdup(reply->brp_un.brp_jid);

	PBSD_FreeReply(reply);
	return return_jobid;
}
