/*
This program is confidential and proprietary to MRJ Technology Solutions
and may not be reproduced, published or disclosed to others without
written authorization from MRJ.

Copyright (c) 1998 MRJ Technology Solutions, Inc. All Rights Reserved.

This program is derived from prior work some of which was originally
written as a joint project between the Numerical Aerospace Simulation
(NAS) Systems Division of NASA Ames Research Center and the National
Energy Research Supercomputer Center (NERSC) of Lawrence Livermore
National Laboratory.
*/
 
static char ident[] = "@(#) $RCSfile: dec_CpyFil.c,v $ $Revision: 2.0 $";

/*
 * decode_DIS_CopyFiles() - decode a Copy Files Dependency Batch Request
 *
 *	This request is used by the server ONLY. 
 *	The batch request structure pointed to by preq must already exist.
 *
 *	Data items are:	string		job id			(may be null)
 *			string		job owner		(may be null)
 *			string		execution user name
 *			string		execution group name	(may be null)
 *			unsigned int	direction	
 *			unsigned int	count of file pairs in set
 *			set of		file pairs:
 *				unsigned int	flag
 *				string		local path name
 *				string		remote path name (may be null)
 */

#include <pbs_config.h>   /* the master config generated by configure */

#include <sys/types.h>
#include <stdlib.h>
#include "libpbs.h"
#include "list_link.h"
#include "server_limits.h"
#include "attribute.h"
#include "credential.h"
#include "batch_request.h"
#include "dis.h"

int decode_DIS_CopyFiles(sock, preq)
	int   sock;
	struct batch_request *preq;
{
	int   pair_ct;
	struct rq_cpyfile *pcf;
	struct rqfpair    *ppair;
	int   rc;

	pcf = &preq->rq_ind.rq_cpyfile;
	CLEAR_HEAD(pcf->rq_pair);
	if ((rc = disrfst(sock, PBS_MAXSVRJOBID, pcf->rq_jobid)) != 0)
		return rc;
	if ((rc = disrfst(sock, PBS_MAXUSER, pcf->rq_owner)) != 0)
		return rc; 
	if ((rc = disrfst(sock, PBS_MAXUSER, pcf->rq_user))  != 0)
		return rc;
	if ((rc = disrfst(sock, PBS_MAXGRPN, pcf->rq_group)) != 0)
		return rc;
	pcf->rq_dir = disrui(sock, &rc);
	if (rc) return rc;
	
	pair_ct = disrui(sock, &rc);
	if (rc) return rc;

	while (pair_ct--) {

		ppair = (struct rqfpair *)malloc(sizeof (struct rqfpair));
		if (ppair == (struct rqfpair *)0)
			return DIS_NOMALLOC;
		CLEAR_LINK(ppair->fp_link);
		ppair->fp_local = 0;
		ppair->fp_rmt   = 0;

		ppair->fp_flag = disrui(sock, &rc);
		if (rc) {
			(void)free(ppair);
			return rc;
		}
		ppair->fp_local = disrst(sock, &rc);
		if (rc) {
			(void)free(ppair);
			return rc;
		}
		ppair->fp_rmt = disrst(sock, &rc);
		if (rc) {
			(void)free(ppair->fp_local);
			(void)free(ppair);
			return rc;
		}
		append_link(&pcf->rq_pair, &ppair->fp_link, ppair);
	}
	return 0;
}
