/*
This program is confidential and proprietary to MRJ Technology Solutions
and may not be reproduced, published or disclosed to others without
written authorization from MRJ.

Copyright (c) 1998 MRJ Technology Solutions, Inc. All Rights Reserved.

This program is derived from prior work some of which was originally
written as a joint project between the Numerical Aerospace Simulation
(NAS) Systems Division of NASA Ames Research Center and the National
Energy Research Supercomputer Center (NERSC) of Lawrence Livermore
National Laboratory.
*/
 
static char ident[] = "@(#) $RCSfile: dec_attropl.c,v $ $Revision: 2.0 $";

/*
 * decode_DIS_attropl() - decode into a list of PBS API "attropl" structures
 *
 *	The space for the attropl structures is allocated as needed.
 *
 *	The first item is a unsigned integer, a count of the
 *	number of attropl entries in the linked list.  This is encoded
 *	even when there are no entries in the list.
 *
 *	Each individual entry is encoded as:
 *		u int	size of the three strings (name, resource, value)
 *			including the terminating nulls, see dec_svrattrl.c
 *		string	attribute name
 *		u int	1 or 0 if resource name does or does not follow
 *		string	resource name (if one)
 *		string  value of attribute/resource
 *		u int	"op" of attrlop (also flag of svrattrl) 
 *
 *	Note, the encoding of a attropl is the same as the encoding of
 *	the pbs_ifl.h structures "attrl" and the server struct svrattrl.
 *	Any one of the three forms can be decoded into any of the three with
 *	the possible loss of the "flags" field (which is the "op" of the
 *	attrlop).
 */

#include <pbs_config.h>   /* the master config generated by configure */

#include <stdlib.h>
#include "libpbs.h"
#include "dis.h"

int decode_DIS_attropl(sock, ppatt)
	int		 sock;
	struct attropl **ppatt;
{
	int		 hasresc;
	int		 i;
	unsigned int	 name_len;
	unsigned int	 numpat;
	struct attropl  *pat;
	struct attropl  *patprior;
	int		 rc;


	numpat = disrui(sock, &rc);
	if (rc) return rc;

	for (i=0; i < numpat; ++i) {

		name_len = disrui(sock, &rc);	/* name_len is unused here */
		if (rc) break;

		pat = malloc(sizeof (struct attropl));
		if (pat == 0)
			return DIS_NOMALLOC;

		pat->next     = (struct attropl *)0;
		pat->name     = (char *)0;
		pat->resource = (char *)0;
		pat->value    = (char *)0;

		pat->name = disrst(sock, &rc);
		if (rc)	break;

		hasresc = disrui(sock, &rc);
		if (rc) break;
		if (hasresc) {
			pat->resource = disrst(sock, &rc);
			if (rc) break;
		}

		pat->value = disrst(sock, &rc);
		if (rc) break;

		pat->op = (enum batch_op)disrui(sock, &rc);
		if (rc) break;

		if (i == 0) {
			/* first one, link to passing in pointer */
			*ppatt = pat;
		} else {
			patprior->next = pat;
		}
		patprior = pat;
	}

	if (rc)
		PBS_free_aopl(pat);
	return rc;
}
