/*
This program is confidential and proprietary to MRJ Technology Solutions
and may not be reproduced, published or disclosed to others without
written authorization from MRJ.

Copyright (c) 1998 MRJ Technology Solutions, Inc. All Rights Reserved.

This program is derived from prior work some of which was originally
written as a joint project between the Numerical Aerospace Simulation
(NAS) Systems Division of NASA Ames Research Center and the National
Energy Research Supercomputer Center (NERSC) of Lawrence Livermore
National Laboratory.
*/
 
static char ident[] = "@(#) $RCSfile: enc_reply.c,v $ $Revision: 2.0 $";

/*
 * encode_DIS_reply() - encode a Batch Protocol Reply Structure
 * 
 * 	batch_reply structure defined in libpbs.h, it must be allocated
 *	by the caller.
 */
 
#include <pbs_config.h>   /* the master config generated by configure */

#include "libpbs.h"
#include "list_link.h"
#include "attribute.h"
#include "dis.h"

int encode_DIS_reply(sock, reply)
	int		    sock;
	struct batch_reply *reply;
{
	int		    ct;
	int		    i;
	struct brp_select  *psel;
	struct brp_status  *pstat;
	svrattrl	   *psvrl;

	int rc;

	/* first encode "header" consisting of protocol type and version */

	if ( (rc = diswui(sock, PBS_BATCH_PROT_TYPE))   ||
	     (rc = diswui(sock, PBS_BATCH_PROT_VER)) )
		return rc;

	/* next encode code, auxcode and choice (union type identifier) */

	if ( (rc = diswsi(sock, reply->brp_code)) 	||
	     (rc = diswsi(sock, reply->brp_auxcode))	||
	     (rc = diswui(sock, reply->brp_choice)))
		return rc;

	switch (reply->brp_choice) {

	    case BATCH_REPLY_CHOICE_NULL:
		break;	/* no more to do */

	    case BATCH_REPLY_CHOICE_Queue:
	    case BATCH_REPLY_CHOICE_RdytoCom:
	    case BATCH_REPLY_CHOICE_Commit:
		if (rc = diswst(sock, reply->brp_un.brp_jid))
			return (rc);
		break;

	    case BATCH_REPLY_CHOICE_Select:

		/* have to send count of number of strings first */

		ct = 0;
		psel = reply->brp_un.brp_select;
		while (psel) {
			++ct;
			psel = psel->brp_next;
		}	
		if (rc = diswui(sock, ct))
			return rc;

		psel = reply->brp_un.brp_select;
		while (psel) {
			/* now encode each string */
			if (rc = diswst(sock, psel->brp_jobid))
				return rc;
			psel = psel->brp_next;
		}
		break;

	    case BATCH_REPLY_CHOICE_Status:

		/* encode "server version" of status structure.
		 *
		 * Server always uses svrattrl form.
		 * Commands decode into their form.
		 *
		 * First need to encode number of status objects and then
		 * the object itself.
		 */

		ct = 0;
		pstat = (struct brp_status *)GET_NEXT(reply->brp_un.brp_status);
		while (pstat) {
			++ct;
			pstat =(struct brp_status *)GET_NEXT(pstat->brp_stlink);
		}
		if (rc = diswui(sock, ct))
			return rc;
		pstat = (struct brp_status *)GET_NEXT(reply->brp_un.brp_status);
		while (pstat) {
			if ( (rc = diswui(sock, pstat->brp_objtype))	||
			     (rc = diswst(sock, pstat->brp_objname)) )
				return rc;

			psvrl = (svrattrl *)GET_NEXT(pstat->brp_attr);
			if (rc = encode_DIS_svrattrl(sock, psvrl))
				return rc;
			pstat =(struct brp_status *)GET_NEXT(pstat->brp_stlink);
		}
		break;

	    case BATCH_REPLY_CHOICE_Text:
		
		/* text reply */

		if (rc = diswcs(sock, reply->brp_un.brp_txt.brp_str,
				      (size_t)reply->brp_un.brp_txt.brp_txtlen))	
			return rc;
		break;

	    case BATCH_REPLY_CHOICE_Locate:

		/* Locate Job Reply */

		if (rc = diswst(sock, reply->brp_un.brp_locate))
			return rc;
		break;

	    case BATCH_REPLY_CHOICE_RescQuery:

		/* Query Resources Reply */

		ct = reply->brp_un.brp_rescq.brq_number;
		if (rc = diswui(sock, ct))
			return rc;
		for (i=0; (i<ct) && (rc == 0); ++i) {
			rc =diswui(sock,*(reply->brp_un.brp_rescq.brq_avail+i));
		}
		if (rc) return rc;
		for (i=0; (i<ct) && (rc == 0); ++i) {
			rc =diswui(sock,*(reply->brp_un.brp_rescq.brq_alloc+i));
		}
		if (rc) return rc;
		for (i=0; (i<ct) && (rc == 0); ++i) {
			rc =diswui(sock,*(reply->brp_un.brp_rescq.brq_resvd+i));
		}
		if (rc) return rc;
		for (i=0; (i<ct) && (rc == 0); ++i) {
			rc =diswui(sock,*(reply->brp_un.brp_rescq.brq_down+i));
		}
		if (rc) return rc;
		break;

	    default:
		return -1;
	}

	return 0;
}
