/*
This program is confidential and proprietary to MRJ Technology Solutions
and may not be reproduced, published or disclosed to others without
written authorization from MRJ.

Copyright (c) 1998 MRJ Technology Solutions, Inc. All Rights Reserved.

This program is derived from prior work some of which was originally
written as a joint project between the Numerical Aerospace Simulation
(NAS) Systems Division of NASA Ames Research Center and the National
Energy Research Supercomputer Center (NERSC) of Lawrence Livermore
National Laboratory.
*/
#include <pbs_config.h>   /* the master config generated by configure */

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "portability.h"
#include "log.h"

static char ident[] = "@(#) $RCSfile: setup_env.c,v $ $Revision: 2.0 $";

/*
 * setup_env - setup the daemon's environment
 *
 *	To provide a "safe and secure" environment, the daemons replace their
 *	inherited one with one from a file.  Each line in the file is
 *	either a comment line, starting with '#' or ' ', or is a line of 
 *	the forms:	variable=value
 *			variable
 *	In the second case the value is obtained from the current environment.
 */

#define PBS_ENVP_STR     64
#define PBS_ENV_CHUNCK 1024

int setup_env(filen)
	char *filen;
{
	char	     buf[256];
	int	     evbufsize = 0;
	FILE	    *efile;
	char        *envbuf;
	static char *envp[PBS_ENVP_STR+1];
	int	     len;
	int	     nstr = 0;
	char	    *pequal;
	char	    *pval;
	extern char **environ;

	envp[0] = (char *)0;
	if ((filen == (char *)0) || (*filen == '\0')) {
		environ = envp;
		return 0;
	}

	efile = fopen(filen, "r");
	if (efile != (FILE *)0) {

	    while (fgets(buf, 255, efile) ) {
		if ((buf[0] != '#') && (buf[0] != ' ') && (buf[0] != '\n')) {
			len = strlen(buf);
			buf[len-1] = '\0';
			if ((pequal = strchr(buf, (int)'=')) == 0) {
				if ((pval = getenv(buf)) == 0)
					continue;
				len += strlen(pval) + 1;
			}
	
			if (evbufsize < len) {
				if ((envbuf = malloc(PBS_ENV_CHUNCK)) == 0)
					goto err;

				evbufsize = PBS_ENV_CHUNCK;
			}
			(void)strcpy(envbuf, buf);
			if (pequal == 0) {
				(void)strcat(envbuf, "=");
				(void)strcat(envbuf, pval);
			}
			envp[nstr++] = envbuf;
			if (nstr == PBS_ENVP_STR)
				goto err;
			envp[nstr] = (char *)0;
			envbuf += len;
			evbufsize -= len;
		}
	    }
	    fclose(efile);
	    environ = envp;
	} else if (errno != ENOENT) {
		goto err;
	}
	return (0);

err:	log_err(-1, "setup_env", "Could not set up the environment");
	return (-1);
}
