/*
This program is confidential and proprietary to MRJ Technology Solutions
and may not be reproduced, published or disclosed to others without
written authorization from MRJ.

Copyright (c) 1998 MRJ Technology Solutions, Inc. All Rights Reserved.

This program is derived from prior work some of which was originally
written as a joint project between the Numerical Aerospace Simulation
(NAS) Systems Division of NASA Ames Research Center and the National
Energy Research Supercomputer Center (NERSC) of Lawrence Livermore
National Laboratory.
*/
#include <pbs_config.h>   /* the master config generated by configure */

#include <sys/types.h>
#include <netdb.h>
#include <netinet/in.h>
#include <memory.h>
#include "portability.h"
#include "server_limits.h"
#include "net_connect.h"
#include "pbs_error.h"

static char ident[] = "@(#) $RCSfile: get_hostaddr.c,v $ $Revision: 2.0 $";

/*
 * get_hostaddr.c - contains functions to provide the internal
 *	internet address for a host and to provide the port
 *	number for a service.
 *
 * get_hostaddr - get internal internet address of a host
 *
 *	Returns a pbs_net_t (unsigned long) containing the network
 *	address in host byte order.  A Null value is returned on
 *	an error.
 */

pbs_net_t get_hostaddr(hostname)
	char *hostname;
{
	static struct in_addr  hostaddr;
	struct hostent *hp;
	extern int h_errno;		/* see gethostbyname() */
	extern int pbs_errno;

	hp = gethostbyname(hostname);
	if (hp == (struct hostent *)0) {
		if (h_errno == TRY_AGAIN)
			pbs_errno = PBS_NET_RC_RETRY;
		else
			pbs_errno = PBS_NET_RC_FATAL;
		return ((pbs_net_t)0);
	}
	
	memcpy((void *)&hostaddr, (void *)hp->h_addr_list[0], hp->h_length);
	return ((pbs_net_t)ntohl(hostaddr.s_addr));
}
