/*
This program is confidential and proprietary to MRJ Technology Solutions
and may not be reproduced, published or disclosed to others without
written authorization from MRJ.

Copyright (c) 1998 MRJ Technology Solutions, Inc. All Rights Reserved.

This program is derived from prior work some of which was originally
written as a joint project between the Numerical Aerospace Simulation
(NAS) Systems Division of NASA Ames Research Center and the National
Energy Research Supercomputer Center (NERSC) of Lawrence Livermore
National Laboratory.
*/
#include <pbs_config.h>   /* the master config generated by configure */

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <errno.h>
#include <netdb.h>
#include "portability.h"
#include "server_limits.h"
#include "net_connect.h"

static char ident[] = "@(#) $RCSfile: net_client.c,v $ $Revision: 2.0 $";

/*
 * client_to_svr - connect to a server
 *
 *	Perform socket/tcp/ip stuff to connect to a server.
 *
 *	Returns: >=0 the socket obtained, or
 *		 PBS_NET_RC_FATAL (-1) if fatal error, just quit, or
 *		 PBS_NET_RC_RETRY (-2) if temp error, should retry
 *
 * Note, the server's host address and port are was chosen as parameters
 * rather than their names to possibly save extra look-ups.  It seems likely
 * that the caller "might" make several calls to the same host or different
 * hosts with the same port.  Let the caller keep the addresses arround
 * rather than look it up each time.
 */

int client_to_svr(hostaddr, port, local_port)
	pbs_net_t	hostaddr;	/* Internet addr  of host */
	unsigned int    port;		/* port to which to connect */
	int	        local_port;	/* not 0 if use local reserved port */
{
	struct sockaddr_in local;
	struct sockaddr_in remote;
	int		 sock;
	unsigned short	 tryport;

	local.sin_family = AF_INET;
	local.sin_addr.s_addr = 0;

	/* get socket					*/

	sock = socket(AF_INET, SOCK_STREAM, 0);
	if (sock < 0) {
		return (PBS_NET_RC_FATAL);
	} else if (sock >= PBS_NET_MAX_CONNECTIONS) {
		(void)close(sock);		/* too many connections */
		return (PBS_NET_RC_RETRY);
	}

	/*	If local privilege port requested, bind to one	*/
	/*	Must be root privileged to do this		*/

	if (local_port) {
		tryport = IPPORT_RESERVED - 1;
		local.sin_port = htons(tryport);

		while (bind(sock,(struct sockaddr *)&local,sizeof(local)) < 0) {
			if ((errno != EADDRINUSE) && 
			    (errno != EADDRNOTAVAIL)) {
				(void)close(sock);
				return (PBS_NET_RC_FATAL);
			} else if (--tryport < (unsigned short)(IPPORT_RESERVED/2)) {
				(void)close(sock);
				return (PBS_NET_RC_RETRY);
			}
			local.sin_port = htons(tryport);
		}
	}
			
/*	connect to specified server host and port	*/

	remote.sin_addr.s_addr = htonl(hostaddr);
	remote.sin_port = htons((unsigned short)port);
	remote.sin_family = AF_INET;
	if (connect(sock, (struct sockaddr *)&remote, sizeof(remote)) < 0) {
		switch (errno) {
		    case EINTR:
		    case EADDRINUSE:
		    case ETIMEDOUT:
		    case ECONNREFUSED:
			close(sock);
			return (PBS_NET_RC_RETRY);
		    default:
			close(sock);
			return (PBS_NET_RC_FATAL);
		}
			
	} else {
		return (sock);
	}
}
