/*
This program is confidential and proprietary to MRJ Technology Solutions
and may not be reproduced, published or disclosed to others without
written authorization from MRJ.

Copyright (c) 1998 MRJ Technology Solutions, Inc. All Rights Reserved.

This program is derived from prior work some of which was originally
written as a joint project between the Numerical Aerospace Simulation
(NAS) Systems Division of NASA Ames Research Center and the National 
Energy Research Supercomputer Center (NERSC) of Lawrence Livermore   
National Laboratory.
*/

#ifndef CHECK_H
#define CHECK_H

#include "server_info.h"
#include "queue_info.h"
#include "job_info.h"

/*
 *	is_ok_to_run_in_queue - check to see if jobs can be run in queue
 */
int is_ok_to_run_queue( queue_info *qinfo );

/*
 *	is_ok_to_run_job - check to see if it ok to run a job on the server
 */
int is_ok_to_run_job( int pbs_sd, server_info *sinfo, queue_info *qinfo, 
							job_info *jinfo);

/*
 *	check_avail_resources - check if there is available resources to run
 */
int check_avail_resources( server_info *sinfo, queue_info *qinfo, 
							job_info *jinfo);
/*
 *	dynamic_avail - find out how much of a resource is available on a 
 */
sch_resource_t dynamic_avail( resource *res );

/* 
 *      check_run_job - function used by job_filter to filter out 
 *                      non-running jobs.
 */
int check_run_job( job_info *job, void *arg );

/*
 *      count_by_user - count the ammount of jobs a user has in a job array
 */
int count_by_user( job_info **jobs, char *user );

/*
 *      check_server_max_group_run - check to see if group is within their 
 *                                   server running limits
 */
int check_server_max_group_run( server_info *sinfo, char *group  );

/*
 *      check_queue_max_user_run - check if the user is within queue
 *                                      user run limits
 */
int check_queue_max_user_run( queue_info *qinfo, char *account );

/*
 *      count_by_group - count number of jobs a group has in job array
 */
int count_by_group( job_info **jobs, char *group );


/*
 *      check_server_max_user_run - check if the user is within server
 *                                      user run limits
 */
int check_server_max_user_run( server_info *sinfo, char *account );

/*
 *      check_queue_max_group_run - check to see if the group is within their
 *                                      queue running limits
 */
int check_queue_max_group_run( queue_info *qinfo, char *group );

/*
 *      will_cross_into_ded_time - check to see if a job would cross into
 *                                 dedicated time
 */
int will_cross_ded_time_boundry( job_info *jinfo );

/*
 *      check_nodes - check to see if there is suficient nodes available to
 *                    run a job.
 */
int check_nodes( int pbs_sd, job_info *jinfo, node_info **ninfo_arr );

/*
 *      is_node_available - determine that there is a node available to run
 *                          the job
 */
int is_node_available( job_info *jinfo, node_info **ninfo_arr );

/*
 *      check_ded_time_queue - check if it is the approprate time to run jobs
 *                             in a dedtime queue
 */
int check_ded_time_queue( queue_info *qinfo );


#endif

