/*
This program is confidential and proprietary to MRJ Technology Solutions
and may not be reproduced, published or disclosed to others without
written authorization from MRJ.

Copyright (c) 1998 MRJ Technology Solutions, Inc. All Rights Reserved.

This program is derived from prior work some of which was originally
written as a joint project between the Numerical Aerospace Simulation
(NAS) Systems Division of NASA Ames Research Center and the National 
Energy Research Supercomputer Center (NERSC) of Lawrence Livermore   
National Laboratory.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "job_info.h"
#include "constant.h"
#include "fairshare.h"
#include "globals.h"
#include "misc.h"
#include "constant.h"
#include "config.h"

static char *ident = "$Id: fairshare.c,v 2.0 1998/10/14 23:16:14 hender Exp $";

/*
 *
 *	add_child - add a group_info to the resource group tree
 *
 *	  ginfo - ginfo to add to the tree
 *	  parent - parent ginfo
 *
 *	returns nothing
 *
 */
void add_child( group_info *ginfo, group_info *parent )
{
  if( parent != NULL )
  {
    ginfo -> sibling = parent -> child;
    parent -> child = ginfo;
    ginfo -> parent = parent;
    ginfo -> resgroup = parent -> cresgroup;
  }
}

/*
 *
 *	add_unknown - add a ginfo to the "unknown" group 
 *
 *	  ginfo - ginfo to add
 *
 *	returns nothing
 *
 */
void add_unknown( group_info *ginfo )
{
  group_info *unknown;		/* ptr to the "unknown" group */

  unknown = find_group_info("unknown", conf.group_root);
  add_child( ginfo, unknown );
  calc_fair_share_perc(unknown -> child, UNSPECIFIED);
}

/* 
 *
 *	find_group_info - recursive function to find a group_info in the 
 *			  resgroup tree
 *
 *	  name - name of the ginfo to find
 *	  root - the root of the current sub-tree
 *
 *	returns the found group_info or NULL
 *
 */
group_info *find_group_info( char *name, group_info *root )
{
  group_info *ginfo;		/* the found group */
  if( root == NULL || !strcmp(name, root -> name) )
    return root;
  
  ginfo = find_group_info( name, root -> sibling );
  if( ginfo == NULL )
    ginfo = find_group_info( name, root -> child );
  
  return ginfo;
}

/*
 *
 *	find_alloc_ginfo - trys to find a ginfo in the fair share tree.  If it
 *			  can not find the ginfo, then allocate a new one and
 *			  add it to the "unknown" group
 *
 *	  name - name of the ginfo to find
 *
 *	returns the found ginfo or the newly allocated ginfo
 *
 */
group_info *find_alloc_ginfo( char *name )
{
  group_info *ginfo;		/* the found group or allocated group */

  ginfo = find_group_info( name, conf.group_root );

  if( ginfo == NULL )
  {
    if( ( ginfo = new_group_info() ) == NULL )
      return NULL;
    
    ginfo-> name = string_dup(name);
    ginfo -> shares = 1;
    add_unknown(ginfo);
  }
  return ginfo;
}

/*
 *
 *	new_group_info - allocate a new group_info struct and initalize it
 *
 *	returns a ptr to the new group_info
 *
 */
group_info *new_group_info()
{
  group_info *new;		/* the new group */

  if( ( new = malloc( sizeof( group_info ) ) ) == NULL )
  {
    perror( "Error allocating memory" );
    return NULL ;
  }

  new -> name = NULL;
  new -> resgroup = UNSPECIFIED;
  new -> cresgroup = UNSPECIFIED;
  new -> shares = UNSPECIFIED;
  new -> percentage = 0.0;
  new -> usage = 1;
  new -> temp_usage = 1;
  new -> parent = NULL;
  new -> sibling = NULL;
  new -> child = NULL;

  return new;
}

/*
 *
 *	parse_group - parse the resource group file
 *
 *	  fname - name of the file
 *
 *	returns success/failure
 *
 *
 *	FORMAT:   name	cresgrp		grpname		shares
 *
 *	  name    - name of user/grp
 *	  cresgrp - resource group of the children of this group (if group)
 *	  grpname - resource group of this user/group
 *	  shares  - the ammount of shares the user/group has in its resgroup
 *		   
 */
int parse_group( char *fname )
{
  group_info *ginfo;		/* ptr to parent group */
  group_info *new_ginfo;	/* used to add each new group */
  char buf[256];		/* used to read each line from the file */
  char *nametok;		/* strtok: name of new group */
  char *grouptok;		/* strtok: parent group name */
  char *cgrouptok;		/* strtok: resgrp of the children of newgrp */
  char *sharestok;		/* strtok: the ammount of shares for newgrp */
  FILE *fp;			/* file pointer to the resource group file */
  char error = 0;		/* boolean: is there an error ? */
  int shares;			/* number of shares for the new group */
  int cgroup;			/* resource group of the children of the grp */
  char *endp;			/* used for strtol() */
  int linenum = 0;		/* current line number in the file */

  if( ( fp = fopen( fname, "r" ) ) == NULL )
  {
    perror( "Warning: resource group file error, fair share will not work");
    return 0;
  }

  while( fgets( buf, 256, fp ) != NULL )
  {
    if( buf[strlen(buf)-1] == '\n' )
      buf[strlen(buf)-1] = '\0';
    linenum++;
    if( !skip_line( buf ) )
    {
      nametok = strtok(buf, "	 ");
      cgrouptok = strtok(NULL, "	 ");
      grouptok = strtok(NULL, " 	");
      sharestok= strtok(NULL,  " 	");

      ginfo = find_alloc_ginfo(grouptok);

      if( ginfo != NULL )
      {
	shares = strtol(sharestok, &endp, 10);
	if( *endp == '\0' )
	{
	  cgroup = strtol(cgrouptok, &endp, 10);
	  if( *endp == '\0' )
	  {
	    if( ( new_ginfo = new_group_info() ) == NULL )
	      return 0;
	    new_ginfo -> name = string_dup( nametok );
	    new_ginfo -> resgroup = ginfo -> cresgroup;
	    new_ginfo -> cresgroup = cgroup;
	    new_ginfo -> shares = shares;
	    add_child( new_ginfo, ginfo );
	  }
	  else
	    error = 1;
	}
	else
	  error = 1;
      }
      else 
      {
	error = 1;
	fprintf(stderr, "Parent ginfo of %s doesnt exist.\n", nametok);
      }
      if( error )
        fprintf(stderr, "resgroup: error on line %d.\n", linenum);
      error = 0;
    }
  }
  return 1;
}

/*
 *
 *	free_group_tree - free the data used by a the fair share tree
 *
 *	  root - root of subtree
 *
 */
void free_group_tree( group_info *root )
{
  if( root == NULL )
    return;
  
  free_group_tree( root -> sibling );
  free_group_tree( root -> child);

  free( root );
}

/*
 *
 *	preload_tree -  load the "root" group into the fair share tree
 *			the "root" group is the entire machine.  Also load 
 *			the "unknown" group.  This group is for any user that
 *			is not specified in the resource group file.
 *
 *	returns success/failure
 *
 */
int preload_tree()
{
  group_info *unknown;		/* pointer to the "unknown" group */
  if( ( conf.group_root = new_group_info() ) == NULL )
    return 0;
  
  if( (unknown = new_group_info() ) == NULL )
    return 0;

  if( ( conf.group_root -> name = malloc( 5 * sizeof(char) ) ) == NULL )
  {
    perror("Memory allocation error");
    return 0;
  }

  if( (unknown -> name = (char *) malloc( 8 * sizeof(char) ) ) == NULL )
  {
    perror("Memory Allocation Error");
    return 0;
  }
  strcpy(conf.group_root -> name, "root");
  conf.group_root -> resgroup = -1;
  conf.group_root -> cresgroup = 0;
  conf.group_root -> percentage = 1.0;

  strcpy(unknown -> name, "unknown");
  unknown -> shares = conf.unknown_shares;
  unknown -> resgroup = 0;
  unknown -> cresgroup = 1;
  unknown -> parent = conf.group_root;
  conf.group_root -> child = unknown;
  return 1;
}

/*
 *
 *	count_shares - count the shares in a resource group
 *		       a resource group is a group_info and all of its 
 *		       siblings
 *
 *	 grp - The start of a sibling chain
 *
 *	returns the number of shares
 *
 */
int count_shares( group_info *grp )
{
  int shares = 0;		/* accumulator to count the shares */
  group_info *cur_grp;		/* the current group in a sibling chain */

  cur_grp = grp;

  while( cur_grp != NULL )
  {
    shares += cur_grp -> shares;
    cur_grp = cur_grp -> sibling;
  }

  return shares;
}

/*
 *
 *	calc_fair_share_perc - walk the fair share group tree and calculate 
 *			       the overall percentage of the machine a user/
 *			       group gets if all usage is equal
 *
 *	  root - the root of the current subtree
 *	  shares - the number of total shares in the group
 *
 *	returns success/failure
 *
 */
int calc_fair_share_perc( group_info *root, int shares )
{
  int cur_shares;		/* total number of shares in the resgrp */

  if( root == NULL )
    return 0;

  if( shares == UNSPECIFIED ) 
    cur_shares = count_shares( root );
  else
    cur_shares = shares;

  root -> percentage = (float) root -> shares / (float) cur_shares * 
		root -> parent -> percentage;

  calc_fair_share_perc( root -> sibling, cur_shares );
  calc_fair_share_perc( root -> child, UNSPECIFIED );
  return 1;
}

/*
 *
 *	test_perc - a debugging function to check if the fair_share 
 *		    percentanges calculated add up to 100%
 *
 *	  root - root of the current subtree
 *
 *	returns total percentage (hopefully 1.0)
 *
 */
float test_perc( group_info *root )
{
  if( root == NULL )
    return 0;
  
 return ( root -> child == NULL ? root -> percentage : 0 ) + test_perc( root -> sibling ) + test_perc( root -> child );
}


/*
 *
 *	update_usage_on_run - update a users usage information when a 
 *			      job is run
 *
 *	  jinfo - the job which just started running
 *
 *	returns nothing
 *
 */
void update_usage_on_run( job_info *jinfo )
{
  jinfo -> ginfo -> temp_usage += calculate_usage_value( jinfo -> resreq );
}

/* 
 *
 *	calculate_usage_value - calcualte a value that represents the usage
 *				information 
 *
 *	  resreq - a resource_req list that holds the resource info
 *
 *	returns the calculated value
 *
 *	NOTE: currently it will only return the number of cpu seconds used.
 *	      This function can be more complicated
 *
 */
usage_t calculate_usage_value( resource_req *resreq )
{
  resource_req *tmp;

  if( resreq != NULL )
  {
    tmp = find_resource_req( resreq, "cput" );
    if( tmp != NULL )
      return tmp -> ammount;
  }

  return 0L;
}

/*
 *
 *	decay_fairshare_tree - decay the usage information kept in the fair
 *			       share tree
 *
 *	  root - the root of the fairshare tree
 *
 *	returns nothing
 *
 */
void decay_fairshare_tree( group_info *root )
{
  if( root == NULL )
    return;
  
  decay_fairshare_tree( root -> sibling );
  decay_fairshare_tree( root -> child );

  root -> usage /= 2;
  if( root -> usage == 0 )
    root -> usage = 1;
}

/*
 *
 *	extract_fairshare - extract the first job from the user with the 
 *			    least percentage / usage ratio
 *
 *	  jobs - array of jobs to search through
 *
 *	return the found job or NULL on error
 *
 */
job_info *extract_fairshare( job_info **jobs )
{
  job_info *max = NULL;		/* job with the max shares / percentage */
  float max_value;		/* max shares / percentage */
  float cur_value;		/* the current shares / percentage value */
  int i;

  if( jobs != NULL )
  {
    max_value = -1;
    for( i = 0; jobs[i] != NULL; i++)
    {
      cur_value=jobs[i] -> ginfo -> percentage / jobs[i] -> ginfo -> temp_usage;
      if( max_value < cur_value && !jobs[i] -> is_running && 
	  !jobs[i] -> can_not_run )
      {
	max = jobs[i];
	max_value = cur_value;
      }
    }
  }

  return max;
}

/*
 *
 *	print_fairshare - print out the fair share tree
 *
 *	  root - root of subtree
 *
 *	returns nothing
 *
 */
void print_fairshare( group_info *root )
{
  if( root == NULL )
    return;
  
  if( root -> child == NULL )
    printf("User: %-10s Grp: %d cgrp: %d Usage %ld Perc: %f\n", root -> name, 
	root -> resgroup, root -> cresgroup, root -> usage, root -> percentage);
  
  print_fairshare( root -> sibling );
  print_fairshare( root -> child );
}

/*
 *
 *	write_usage - write the usage information to the usage file
 *		      This fuction uses a recursive helper function
 *
 *
 *	returns success/failure
 *
 */
int write_usage()
{
  FILE *fp;		/* file pointer to usage file */

  if( ( fp = fopen(USAGE_FILE, "wb") ) == NULL )
  {
    perror("Error opening file " USAGE_FILE);
    return 0;
  }

  rec_write_usage(conf.group_root, fp);
  return 1;
}

/*
 *
 *	rec_write_usage - recursive helper function which will write out all
 *			  the group_info structs of the resgroup tree
 *
 *	  root - the root of the current subtree
 *	  fp   - the file to write the ginfo out to
 *
 *	returns nothing
 *
 */
void rec_write_usage( group_info *root, FILE *fp )
{
  struct group_node_usage grp;		/* used to write out usage info */

  if( root == NULL )
    return;

  if( root -> usage != 1 )		/* usage defaults to 1 */
  {
    strcpy(grp.name, root -> name);
    grp.usage = root -> usage;

    fwrite(&grp, sizeof(struct group_node_usage), 1, fp);
  }

  rec_write_usage(root -> sibling, fp);
  rec_write_usage(root -> child, fp);
}

/*
 *
 *	read_usage - read the usage information and load it into the 
 *		     resgroup tree.
 *
 *	returns success/failure
 *
 */
int read_usage( )
{
  FILE *fp;				/* file pointer to usage file */
  struct group_node_usage grp;		/* struct used to read in usage info */
  group_info *ginfo;			/* ptr to current group usage */


  if( ( fp = fopen(USAGE_FILE, "r") ) == NULL )
  {
    perror("Can not open file " USAGE_FILE);
    return 0;
  }

  while( fread(&grp, sizeof( struct group_node_usage ), 1, fp ) )
  {
    ginfo = find_alloc_ginfo(grp.name);
    if( ginfo != NULL )
      ginfo -> usage = grp.usage;
  }
  return 1;
}
