/*
This program is confidential and proprietary to MRJ Technology Solutions
and may not be reproduced, published or disclosed to others without
written authorization from MRJ.

Copyright (c) 1998 MRJ Technology Solutions, Inc. All Rights Reserved.

This program is derived from prior work some of which was originally
written as a joint project between the Numerical Aerospace Simulation
(NAS) Systems Division of NASA Ames Research Center and the National 
Energy Research Supercomputer Center (NERSC) of Lawrence Livermore   
National Laboratory.
*/

#include <stdio.h>
#include "globals.h"
#include "constant.h"
#include "sort.h"

static char *ident = "$Id: globals.c,v 2.0 1998/10/14 23:16:40 hender Exp $";

/*
 *	res_to_check[] - a list of resources which controls what resources
 *			 are checked by the scheduler.
 *
 *	Format: resource_name comment_message debug_message
 *
 *	resource_name: name the server knows the resource as
 *	comment_message: what the comment should be set to if there is not
 *			 enough of this resource available
 *	debug_message: what should be logged as the debug message if there 
 *		       is not enough of this resource available
 *
 */

const struct rescheck res_to_check[] = 
{ 
  {"mem", "Not Running: Not enough memory available", "Not enough memory available"},
  { "ncpus", "Not Running: Not enough cpus available", "Not enough cpus available"}
};

/*
 *
 *	sorting_info[] - holds information about all the different ways you
 *			 can sort the jobs
 *
 *	Format: { sort_type, config_name, cmp_func_ptr }
 *
 *	  sort_type    : an element from the enum sort_type
 *	  config_name  : the name which appears in the scheduling policy config 
 *		         file (sched_config)
 *	  cmp_func_ptr : function pointer the qsort compare function
 *			 (located in sort.c)
 *
 */

const struct sort_info sorting_info[] =
{ 
  {NO_SORT, "no_sort", NULL},
  {SHORTEST_JOB_FIRST, "shortest_job_first", cmp_job_cput_asc},
  {LONGEST_JOB_FIRST, "longest_job_first", cmp_job_cput_dsc},
  {SMALLEST_MEM_FIRST, "smallest_memory_first", cmp_job_mem_asc},
  {LARGEST_MEM_FIRST, "largest_memory_first", cmp_job_mem_dsc},
  {HIGH_PRIORITY_FIRST, "high_priority_first", cmp_job_prio_dsc},
  {LOW_PRIORITY_FIRST, "low_priority_first", cmp_job_prio_asc},
  {LARGE_WALLTIME_FIRST, "large_walltime_first", cmp_job_walltime_dsc},
  {SHORT_WALLTIME_FIRST, "short_walltime_first", cmp_job_walltime_asc},
  {FAIR_SHARE, "fair_share", cmp_fair_share},
  {MULTI_SORT, "multi_sort", multi_sort}
};

/*
 *	res_to_get - resources to get from each nodes mom
 */
const char *res_to_get[] = 
{
  "ncpus",		/* number of CPUS */
  "arch",		/* the architecture of the machine */
  "physmem",		/* the ammount of physical memory */
  "loadave",		/* the current load average */
  "max_load",		/* static max_load value */
  "ideal_load"		/* static ideal_load value */	
};

/* number of indicies in the res_to_check array */
const int num_res = sizeof( res_to_check ) / sizeof( struct rescheck );

/* number of indicies in the sorting_info array */
const int num_sorts = sizeof( sorting_info ) / sizeof( struct sort_info );

/* number of indicies in the res_to_get array */
const int num_resget = sizeof( res_to_get ) / sizeof( char * );


struct config conf;
struct status cstat;
