/*
This program is confidential and proprietary to MRJ Technology Solutions
and may not be reproduced, published or disclosed to others without
written authorization from MRJ.

Copyright (c) 1998 MRJ Technology Solutions, Inc. All Rights Reserved.

This program is derived from prior work some of which was originally
written as a joint project between the Numerical Aerospace Simulation
(NAS) Systems Division of NASA Ames Research Center and the National 
Energy Research Supercomputer Center (NERSC) of Lawrence Livermore   
National Laboratory.
*/

#ifndef NODE_INFO_H
#define NODE_INFO_H

#include "data_types.h"
#include <pbs_ifl.h>

/*
 *      query_nodes - query all the nodes associated with a server
 */
node_info **query_nodes( int pbs_sd, server_info *sinfo );

/*
 *      query_node_info - collect information from a batch_status and
 *                        put it in a node_info struct for easier access
 */
node_info *query_node_info( struct batch_status *node, server_info *sinfo );

/*
 *      free_nodes - free all the nodes in a node_info array
 */
void free_nodes( node_info **ninfo_arr );


/*
 *      new_node_info - allocates a new node_info
 */
node_info *new_node_info();

/*
 *      free_node_info - frees memory used by a node_info
 */
void free_node_info( node_info *ninfo );

/*
 *      set_node_state - set a node state
 */
int set_node_state( node_info *ninfo, char *state );

/*
 *      talk_with_mom - talk to mom and get resources
 */
int talk_with_mom( node_info *ninfo );

/*
 *      node_filter - filter a node array and return a new filterd array
 */
node_info **node_filter( node_info **nodes, int size,
                         int (*filter_func) (node_info*, void*), void *arg );


/*
 *      is_node_timeshared - check if a node is timeshared
 */
int is_node_timeshared( node_info *node, void *arg );

/*
 *      find_best_node - find the best node to run a job in a node array
 */
node_info *find_best_node( job_info *jinfo, node_info **ninfo_arr );

/*
 *      find_node_info - find a node in the node array
 */
node_info *find_node_info( char *nodename, node_info **ninfo_arr );

/*
 *      print_node - print all the information in a node.  Mainly used for
 *                   debugging purposes
 */
void print_node( node_info *ninfo, int brief );


#endif
