/*
This program is confidential and proprietary to MRJ Technology Solutions
and may not be reproduced, published or disclosed to others without
written authorization from MRJ.

Copyright (c) 1998 MRJ Technology Solutions, Inc. All Rights Reserved.

This program is derived from prior work some of which was originally
written as a joint project between the Numerical Aerospace Simulation
(NAS) Systems Division of NASA Ames Research Center and the National 
Energy Research Supercomputer Center (NERSC) of Lawrence Livermore   
National Laboratory.
*/

#include <stdio.h>
#include <stdlib.h>
#include "prev_job_info.h"
#include "job_info.h"

static char *ident = "$Id: prev_job_info.c,v 2.0 1998/10/14 23:17:07 hender Exp $";

/*
 *
 *	create_prev_job_info - create the prev_job_info array from an array 
 *				of jobs
 *
 *	  jinfo_arr - job_info array
 *	  size - size of jinfo_arr or UNSPECIFIED if unknown
 *
 *	returns new prev_job_array
 *
 *	NOTE: jinfo_arr is modified
 *
 */
prev_job_info *create_prev_job_info( job_info **jinfo_arr, int size )
{
  prev_job_info *new;		/* new prev_job_info array */
  int local_size;		/* the size of the array */
  int i;

  if( size == UNSPECIFIED )
  {
    for( i = 0; jinfo_arr[i] != NULL; i++ )
	; 
    
    local_size = i;
  }
  else 
    local_size = size;

  if( ( new = (prev_job_info *) malloc( sizeof( prev_job_info)  * local_size ) ) == NULL )
  {
    fprintf(stderr, "Error allocating memory\n");
    return NULL;
  }

  for( i = 0; jinfo_arr[i] != NULL; i++ )
  {
    new[i].name = jinfo_arr[i] -> name;
    new[i].resused = jinfo_arr[i] -> resused;
    new[i].account = jinfo_arr[i] -> account;
    new[i].ginfo = jinfo_arr[i] -> ginfo;

    /* so the memory is not freed at the end of the scheduling cycle */
    jinfo_arr[i] -> name = NULL;
    jinfo_arr[i] -> resused = NULL;
    jinfo_arr[i] -> account = NULL;
  }

  return new;
}

/*
 *
 *	free_prev_job_info - free a prev_job_info struct
 *
 *	  pjinfo - prev_job_info to free
 *
 *	returns nothing
 *
 */
void free_prev_job_info( prev_job_info *pjinfo )
{
  if( pjinfo -> name != NULL )
    free(pjinfo -> name);
  
  if( pjinfo -> account != NULL )
    free(pjinfo -> account);
  
  free_resource_req_list( pjinfo -> resused );
}

/*
 *
 *	free_pjobs - free a list of prev_job_info structs
 *
 *	  pjinfo_arr - array of prev_job_info structs to free
 *
 *	returns nothing
 *
 */
void free_pjobs( prev_job_info *pjinfo_arr, int size )
{
  int i;

  if( pjinfo_arr == NULL )
    return;
  
  for( i = 0; i < size; i++ )
    free_prev_job_info( &pjinfo_arr[i] );
  
  free(pjinfo_arr);
}
