/*
This program is confidential and proprietary to MRJ Technology Solutions
and may not be reproduced, published or disclosed to others without
written authorization from MRJ.

Copyright (c) 1998 MRJ Technology Solutions, Inc. All Rights Reserved.

This program is derived from prior work some of which was originally
written as a joint project between the Numerical Aerospace Simulation
(NAS) Systems Division of NASA Ames Research Center and the National 
Energy Research Supercomputer Center (NERSC) of Lawrence Livermore   
National Laboratory.
*/

#ifndef QUEUE_INFO_H
#define QUEUE_INFO_H

#include <pbs_ifl.h>
#include "data_types.h"


/*
 *
 *      query_queues - creates an array of queue_info structs which contain
 *                      an array of jobs
 */
queue_info **query_queues( int pbs_sd, server_info *sinfo );

/*
 *      query_queue_info - collects information from a batch_status and
 *                         puts it in a queue_info struct for easier access
 */
queue_info *query_queue_info( struct batch_status *queue, struct server_info *sinfo );

/*
 *      new_queue_info - allocate and initalize a new queue_info struct
 */
queue_info *new_queue_info();

/*
 *      print_queue_info - print all information in a queue_info struct
 */
void print_queue_info( queue_info *qinfo, char brief, char deep );

/*
 *      free_queues - frees the memory for an array
 */
void free_queues( queue_info **qinfo, char free_jobs_too);

/*
 *      update_queue_on_run - update the information kept in a qinfo structure
 *                              when a job is run
 */ 
void update_queue_on_run(queue_info *qinfo, job_info *jinfo);

/*
 *      free_queue_info - free space used by a queue info struct
 */
void free_queue_info( queue_info *qinfo );

/*
 *
 *        translate_queue_fail_code - translate the failure code of
 *                      is_ok_to_run_in_queue into log and comment_msg messages
 */
void translate_queue_fail_code( int pbs_sd,
                                char *comment_msg_msg, char *log_msg );


#endif /* QUEUE_INFO_H */

