/*
This program is confidential and proprietary to MRJ Technology Solutions
and may not be reproduced, published or disclosed to others without
written authorization from MRJ.

Copyright (c) 1998 MRJ Technology Solutions, Inc. All Rights Reserved.

This program is derived from prior work some of which was originally
written as a joint project between the Numerical Aerospace Simulation
(NAS) Systems Division of NASA Ames Research Center and the National 
Energy Research Supercomputer Center (NERSC) of Lawrence Livermore   
National Laboratory.
*/

#include <stdio.h>
#include "data_types.h"
#include "sort.h"
#include "job_info.h"
#include "misc.h"
#include "globals.h"
#include "fairshare.h"

static char *ident = "$Id: sort.c,v 2.0 1998/10/14 23:17:31 hender Exp $";


/*
 *	sort.c
 *
 *	This file will hold the compare functions used by qsort
 *	to sort the jobs
 *
 */


/*
 *
 *      cmp_queue_prio_dsc - sort queues in decending priority
 *
 */
int cmp_queue_prio_dsc( const void *q1, const void *q2 )
{
  if( (*(queue_info **) q1) -> priority < (*(queue_info **) q2) -> priority )
    return 1;
  else if((*(queue_info **) q1) -> priority > (*(queue_info **) q2) -> priority)
    return -1;
  else
    return 0;
}

/*
 *
 *      cmp_queue_prio_asc - sort queues by ascending priority
 *
 */
int cmp_queue_prio_asc( const void *q1, const void *q2 )
{
  if( (*(queue_info **) q1) -> priority < (*(queue_info **) q2) -> priority )
    return -1;
  else if((*(queue_info **) q1) -> priority > (*(queue_info **) q2) -> priority)
    return 1;
  else
    return -1;
}

/*
 *
 *      cmp_job_walltime_asc - sort jobs by requested walltime 
 *					in ascending order.
 *
 */
int cmp_job_walltime_asc(const void *j1, const void *j2)
{
  resource_req *req1, *req2;

  req1 = find_resource_req( (*(job_info**) j1) -> resreq, "walltime");
  req2 = find_resource_req( (*(job_info**) j2) -> resreq, "walltime");

  if( req1 != NULL && req2 != NULL )
  {
    if( req1 -> ammount < req2 -> ammount )
      return -1;
    else if( req1 -> ammount == req2 -> ammount )
      return 0;
    else
      return 1;
  }
  else
    return 0;
}

/*
 *
 *      cmp_job_walltime_dsc - sort jobs by requested walltime 
 *					in ascending order.
 *
 */
int cmp_job_walltime_dsc(const void *j1, const void *j2)
{
  resource_req *req1, *req2;

  req1 = find_resource_req( (*(job_info**) j1) -> resreq, "walltime");
  req2 = find_resource_req( (*(job_info**) j2) -> resreq, "walltime");

  if( req1 != NULL && req2 != NULL )
  {
    if( req1 -> ammount < req2 -> ammount )
      return 1;
    else if( req1 -> ammount == req2 -> ammount )
      return 0;
    else
      return -1;
  }
  else
    return 0;
}

/*
 *
 *      cmp_job_cput_asc - sort jobs by requested cput time in ascending order.
 *
 */
int cmp_job_cput_asc(const void *j1, const void *j2)
{
  resource_req *req1, *req2;

  req1 = find_resource_req( (*(job_info**) j1) -> resreq, "cput");
  req2 = find_resource_req( (*(job_info**) j2) -> resreq, "cput");

  if( req1 != NULL && req2 != NULL )
  {
    if( req1 -> ammount < req2 -> ammount )
      return -1;
    else if( req1 -> ammount == req2 -> ammount )
      return 0;
    else
      return 1;
  }
  else
    return 0;
}

/*
 *
 *      cmp_job_cput_dsc - sort jobs by requested cput time in descending order.
 *
 */
int cmp_job_cput_dsc(const void *j1, const void *j2)
{
  resource_req *req1, *req2;

  req1 = find_resource_req( (*(job_info**) j1) -> resreq, "cput");
  req2 = find_resource_req( (*(job_info**) j2) -> resreq, "cput");

  if( req1 != NULL && req2 != NULL )
  {
    if( req1 -> ammount < req2 -> ammount )
      return 1;
    else if( req1 -> ammount == req2 -> ammount )
      return 0;
    else
      return -1;
  }
  else
    return 0;
}

/*
 *
 *      cmp_job_mem_asc - sort jobs by requested mem time in ascending order.
 *
 */
int cmp_job_mem_asc(const void *j1, const void *j2)
{
  resource_req *req1, *req2;

  req1 = find_resource_req( (*(job_info**) j1) -> resreq, "mem");
  req2 = find_resource_req( (*(job_info**) j2) -> resreq, "mem");

  if( req1 != NULL && req2 != NULL )
  {
    if( req1 -> ammount < req2 -> ammount )
      return -1;
    else if( req1 -> ammount == req2 -> ammount )
      return 0;
    else
      return 1;
  }
  else
    return 0;
}

/*
 *
 *      cmp_job_mem_dsc - sort jobs by requested mem time in descending order.
 *
 */
int cmp_job_mem_dsc(const void *j1, const void *j2)
{
  resource_req *req1, *req2;

  req1 = find_resource_req( (*(job_info**) j1) -> resreq, "mem");
  req2 = find_resource_req( (*(job_info**) j2) -> resreq, "mem");

  if( req1 != NULL && req2 != NULL )
  {
    if( req1 -> ammount < req2 -> ammount )
      return 1;
    else if( req1 -> ammount == req2 -> ammount )
      return 0;
    else
      return -1;
  }
  else
    return 0;
}

/*
 *
 *      cmp_job_prio_asc - sort jobs by ascending priority
 *
 */
int cmp_job_prio_asc( const void *j1, const void *j2 )
{
  if( (*(job_info **) j1) -> priority < (*(job_info **) j2) -> priority )
    return -1;
  else if((*(job_info **) j1) -> priority > (*(job_info **) j2) -> priority)
    return 1;
  else
    return 0;
}

/*
 *
 *      cmp_job_prio_dsc - sort jobs by descending priority
 *
 */
int cmp_job_prio_dsc( const void *j1, const void *j2 )
{
  if( (*(job_info **) j1) -> priority < (*(job_info **) j2) -> priority )
    return 1;
  else if((*(job_info **) j1) -> priority > (*(job_info **) j2) -> priority)
    return -1;
  else
    return 0;
}

/*
 *
 *	cmp_fair_share - compare on fair share percentage only.  
 *			 This is for strict priority.
 *
 */
int cmp_fair_share( const void *j1, const void *j2 )
{
  group_info *g1, *g2;

  g1 = (*(job_info **) j1) -> ginfo;
  g2 = (*(job_info **) j2) -> ginfo;

  if( g1 -> percentage > g2 -> percentage )
    return 1;
  else if( g1 -> percentage == g2 -> percentage )
    return 0;
  else
    return 1;
}

/* multi keyed sorting 
 * call compare function to sort for the first key
 * if the two keys are equal, call the compare funciton for the second key
 * repeat for all keys
 */


/*
 *	multi_sort - a multi keyed sorting compare function
 */	
int multi_sort( const void *j1, const void *j2 )
{
  int ret = 0;
  int i;

  for(i = 1; i <= num_sorts && ret == 0 && cstat.sort_by[i].sort != NO_SORT;i++)
    ret = cstat.sort_by[i].cmp_func(j1, j2);
  
  return ret;
}

/*
 *
 *	cmp_sort - entrypoint into job sort used by qsort
 *
 */
int cmp_sort( const void *v1, const void *v2 )
{
  /* sort on the internal scheduler priority (used for starvation code) */
  if((*(job_info **) v1) -> sch_priority < (*(job_info **) v2) -> sch_priority)
    return 1;
  else if((*(job_info **) v1)->sch_priority > (*(job_info **) v2)->sch_priority)
    return -1;
  else
    return cstat.sort_by -> cmp_func(v1, v2);
}
