/*
This program is confidential and proprietary to MRJ Technology Solutions
and may not be reproduced, published or disclosed to others without
written authorization from MRJ.

Copyright (c) 1998 MRJ Technology Solutions, Inc. All Rights Reserved.

This program is derived from prior work some of which was originally
written as a joint project between the Numerical Aerospace Simulation
(NAS) Systems Division of NASA Ames Research Center and the National 
Energy Research Supercomputer Center (NERSC) of Lawrence Livermore   
National Laboratory.
*/

#ifndef SORT_H
#define SORT_H

/*
 *
 *      cmp_queue_prio_dsc - compare function used by qsort to sort queues
 *                           by decending priority
 *
 */
int cmp_queue_prio_dsc( const void *q1, const void *q2 );

/*
 *
 *      cmp_queue_prio_asc - compare function used by qsort to sort queues
 *                           by ascending priority
 *
 */
int cmp_queue_prio_asc( const void *q1, const void *q2 );

/*
 *      cmp_job_walltime_asc - sort jobs by requested walltime
 *                                      in ascending order.
 */
int cmp_job_walltime_asc(const void *j1, const void *j2);

/*
 *      cmp_job_walltime_dsc - sort jobs by requested walltime
 *                                      in ascending order.
 */
int cmp_job_walltime_dsc(const void *j1, const void *j2);


/*
 *
 *      cmp_job_cput_asc - compare function used by qsort to sort the jobs by
 *                         requested cput time in ascending order.
 *
 */
int cmp_job_cput_asc(const void *j1, const void *j2);

/*
 *
 *      cmp_job_cput_dsc - compare function used by qsort to sort the jobs by
 *                         requested cput time in descending order.
 *
 */
int cmp_job_cput_dsc(const void *j1, const void *j2);

/*
 *
 *      cmp_job_mem_asc -  compare function used by qsort to sort the jobs by
 *                         requested mem time in ascending order.
 *
 */
int cmp_job_mem_asc(const void *j1, const void *j2);

/*
 *
 *      cmp_job_mem_dsc -  compare function used by qsort to sort the jobs by
 *                         requested mem time in descending order.
 *
 */
int cmp_job_mem_dsc(const void *j1, const void *j2);

/*
 *
 *      cmp_job_prio_asc - compare function used by qsort to sort jobs
 *                         by ascending priority
 *
 */
int cmp_job_prio_asc( const void *j1, const void *j2 );

/*
 *
 *      cmp_job_prio_dsc - compare function used by qsort to sort jobs
 *                         by descending priority
 *
 */
int cmp_job_prio_dsc( const void *j1, const void *j2 );

/*
 *      cmp_fair_share - compare function for the fair share algorithm
 */
int cmp_fair_share( const void *j1, const void *j2 );


/*
 *      multi_sort - a multi keyed sortint compare function
 */     
int multi_sort( const void *j1, const void *j2 );

/*
 *
 *      cmp_sort - entrypoint into job sort used by qsort 
 */
int cmp_sort( const void *v1, const void *v2 );


#endif
