/*
This program is confidential and proprietary to MRJ Technology Solutions
and may not be reproduced, published or disclosed to others without
written authorization from MRJ.

Copyright (c) 1998 MRJ Technology Solutions, Inc. All Rights Reserved.

This program is derived from prior work some of which was originally
written as a joint project between the Numerical Aerospace Simulation
(NAS) Systems Division of NASA Ames Research Center and the National 
Energy Research Supercomputer Center (NERSC) of Lawrence Livermore   
National Laboratory.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <pbs_error.h>
#include <pbs_ifl.h>
#include <log.h>
#include "state_count.h"
#include "constant.h"
#include "misc.h"

static char *ident = "$Id: state_count.c,v 2.0 1998/10/14 23:17:36 hender Exp $";


/*
 *
 *	print_state_count - print out a state_count struct
 *
 *	  sc - the struct to print
 *
 *	returns nothing
 *
 */
void print_state_count( state_count *sc )
{
  printf("running: %d\n", sc -> running);
  printf("queued: %d\n", sc -> queued);
  printf("transit: %d\n", sc -> transit);
  printf("exiting: %d\n", sc -> exiting);
  printf("held: %d\n", sc -> held);
  printf("waiting: %d\n", sc -> waiting);
  printf("total: %d\n", sc -> total);
}

/*
 *
 *	init_state_count - initalize state count struct
 *
 *	  sc - the struct to initalize
 *
 *	returns nothing
 *
 */
void init_state_count( state_count *sc )
{
  sc -> running = 0;
  sc -> queued = 0;
  sc -> transit = 0;
  sc -> exiting = 0;
  sc -> held = 0;
  sc -> waiting = 0;
  sc -> total = 0;
}

/*
 *
 *	count_states - count the jobs in each state and set the state counts
 *
 *	  jobs - array of jobs
 *	  sc   - state count structure passed by reference
 *
 *	returns nothing
 *
 */
void count_states( job_info **jobs, state_count *sc )
{
  int i;

  if( jobs  != NULL )
  {
    for( i = 0; jobs[i] != NULL; i++ )
    {
      if( jobs[i] -> is_queued )
	sc -> queued++;
      else if( jobs[i] -> is_running )
	sc -> running++;
      else if( jobs[i]-> is_transit )
	sc -> transit++;
      else if( jobs[i] -> is_exiting )
	sc -> exiting++;
      else if( jobs[i] -> is_held )
	sc -> held++;
      else if( jobs[i] -> is_waiting )
	sc -> waiting++;
      else
	log( PBSEVENT_JOB, PBS_EVENTCLASS_JOB, jobs[i] -> name, "Job in unknown state");
    }
  }

  sc -> total = sc -> queued + sc -> running + sc -> transit + sc -> exiting + sc -> held + sc -> waiting;

}

/*
 *
 *	total_states - add the states from sc2 to the states in sc1
 *		       i.e. sc1 += sc2
 *
 *	  sc1 - the accumlator
 *	  sc2 - what is being totaled
 *
 *	returns nothing
 *
 */
void total_states( state_count *sc1, state_count *sc2 )
{
  sc1 -> running += sc2 -> running;
  sc1 -> queued += sc2 -> queued;
  sc1 -> held += sc2 -> held;
  sc1 -> waiting += sc2 -> waiting;
  sc1 -> exiting += sc2 -> exiting;
  sc1 -> transit += sc2 -> transit;
  sc1 -> total += sc2 -> total;
}

