#############################################################################
##                           PBS CU-scheduler
##
## CU-sched-local.tcl: queuing system definition file
##
## (c) 1998  Dirk Grunwald and Artur Klauser, 
##           University of Colorado at Boulder,
##           [grunwald,klauser]@cs.colorado.edu
##
## $Id: CU-sched-foobar.tcl,v 2.0 1998/10/14 23:19:36 hender Exp $
## $Log: CU-sched-foobar.tcl,v $
## Revision 2.0  1998/10/14 23:19:36  hender
## PBS Release 2.0
##
## Revision 1.1  1998/08/06 21:20:13  hender
## PR #1234 - Dec Alpha mods and scheduler changes from U Colorado
##
## Revision 1.1  1998/08/05 22:20:58  grunwald
## Sample TCL based schedulers.
##
## Revision 1.2  1998/02/16 19:25:16  klauser
## fixed a few typos and qinfo problems
##
## Revision 1.1  1998/02/12 01:55:35  klauser
## CU-scheduler implementation revised for better computational complexity;
## easier portable to new scheduler hosts: all changes are localized in one
## file now;
##
#############################################################################

#############################################################################
## name of PBS server machine
#############################################################################
set pbs_server "foobar"

#############################################################################
## cpu_pool defines the schedulable machines
## it is subgrouped into clusters, where each machine is member of only
## one cluster; clusters have flags associated with them which determine
## their scheduling behavior; scheduling priority in the cpu_pool is
## top to bottom, i.e. machines defined first will be used first;
## scheduling priority within "onlyidle" clusters is least loaded to
## most loaded;
## flags:
##   onlyidle ...... schedule on this machine only if it is idle
##   noschedule .... do not schedule on these machine (temporarily disabled)
#############################################################################
set cpu_pool { \
  { \
    {cluster miatas} \
    {comment {The miata systems}} \
    {flags {}} \
    {machines {mx6-0 mx6-1 mx6-2 mx6-3 mx6-4 mx6-5 mx6-6 mx6-7 \
               mx6-8 mx6-9 mx6-10 mx6-11 mx6-12 mx6-13 mx6-14 mx6-15}} \
  } \
  { \
    {cluster others} \
    {comment {Other systems owned by SRL}} \
    {flags {}} \
    {machines {bdd blit shifty car cdr frob mumble \
               raboof rack ragtop torque twoseater }} \
  } \
  { \
    {cluster interactive} \
    {comment {Interactive systems}} \
    {flags {onlyidle}} \
    {machines {disable foobar}} \
  } \
  { \
    {cluster avalanches} \
    {comment {The shared avalanche pool}} \
    {flags {onlyidle}} \
    {machines {av0 av1 av2 av3 av4 av5 av6 av7 av8 av9 \
               av10 av11 av12 av13 av14 av15 av16 av17 av18 \
               av20 av21 av22 av23 av24 av25 av26 av27 av28 av29 \
               av30 av31 av32}} \
  } \
  { \
    {cluster whimpy} \
    {comment {The old whimpy SRL machines}} \
    {flags {noschedule}} \
    {machines {munge widget diddle hosed}} \
  } \
  { \
    {cluster bigmem} \
    {comment {The bigmem SRL machines}} \
    {flags {noschedule}} \
    {machines {bdd car foobar}} \
  } \
}

#############################################################################
## special queues are used for queueing/routing only - no execution
#############################################################################

set special_queue_names "funnel"

#############################################################################
## User job limits
#############################################################################

## default job limit for users without specified limit
set user_max_jobs_default 100

## specific user job limit
set user_max_jobs(mfarrens) 5
set user_max_jobs(lindsay) 15
set user_max_jobs(seidl) 10
set user_max_jobs(arp) 15
set user_max_jobs(blair) 0
set user_max_jobs(numenor) 0
set user_max_jobs(iris) 5
set user_max_jobs(gianluca) 5
# set user_max_jobs(bobbie) 20

set PBS /tools/srl/PBS

set disable_queues_if_no_reply 1

set debug 1
set debug2 0
