#############################################################################
##                           PBS CU-scheduler
##
## CU-sched-init.tcl: source common routines and variables
##
## (c) 1998  Dirk Grunwald and Artur Klauser, 
##           University of Colorado at Boulder,
##           [grunwald,klauser]@cs.colorado.edu
##
## $Id: CU-sched-init.tcl,v 2.0 1998/10/14 23:19:38 hender Exp $
## $Log: CU-sched-init.tcl,v $
## Revision 2.0  1998/10/14 23:19:38  hender
## PBS Release 2.0
##
## Revision 1.1  1998/08/06 21:20:16  hender
## PR #1234 - Dec Alpha mods and scheduler changes from U Colorado
##
## Revision 1.1  1998/08/05 22:20:58  grunwald
## Sample TCL based schedulers.
##
## Revision 1.2  1998/02/16 19:25:17  klauser
## fixed a few typos and qinfo problems
##
## Revision 1.1  1998/02/12 01:55:36  klauser
## CU-scheduler implementation revised for better computational complexity;
## easier portable to new scheduler hosts: all changes are localized in one
## file now;
##
#############################################################################

set hostname [lindex [split [exec hostname] .] 0]

source "/usr/spool/PBS/sched_priv/CU-sched-common.tcl"
source "/usr/spool/PBS/sched_priv/CU-sched-$hostname.tcl"

pbsconnect $pbs_server

#############################################################################
## pre-process cpu_pool data structure
#############################################################################
if { [info exists cpu_flags] } { unset cpu_flags }
if { [info exists cpu_cluster] } { unset cpu_cluster }
foreach cluster $cpu_pool {
  set flags [getval "flags" $cluster]
  set cname [getval "cluster" $cluster]
  if { ! [info exists quiet_init] } {
    puts -nonewline "found cluster $cname ($flags):"
  }
  foreach machine [getval "machines" $cluster] {
    if { ! [info exists quiet_init] } {
      puts -nonewline " $machine"
    }
    if { [lsearch -exact $flags "noschedule"] != -1 } {
      ## machines in noschedule clusters are dropped immediately
      continue
    }
    if { [info exists cpu_cluster($machine)] } {
      if { ! [info exists quiet_init] } {
        puts ""
        puts "ERROR: machine $machine is in two clusters: $cpu_cluster($machine), $cname"
        exit 1
      }
    }
    set cpu_flags($machine) $flags
    set cpu_cluster($machine) $cname
  }
  if { ! [info exists quiet_init] } {
    puts ""
  }
}
unset cluster
unset machine
unset flags
unset cname

## clear the user-host cache
if {[info exists uh_cache]} {
  unset uh_cache
}

## invoke scheduler body once right away
if { ! [info exists is_qinfo]} {
  source "/usr/spool/PBS/sched_priv/CU-sched-body.tcl"
}
