/*
This program is confidential and proprietary to MRJ Technology Solutions
and may not be reproduced, published or disclosed to others without
written authorization from MRJ.

Copyright (c) 1998 MRJ Technology Solutions, Inc. All Rights Reserved.

This program is derived from prior work some of which was originally
written as a joint project between the Numerical Aerospace Simulation
(NAS) Systems Division of NASA Ames Research Center and the National
Energy Research Supercomputer Center (NERSC) of Lawrence Livermore
National Laboratory.
*/




/*
 * node_func.c - various functions dealing with nodes, properties and
 * 		 the server's global node list (pbsnlist) and node list
 *		 variables, svr_totnodes, svr_clnodes, svr_tsnodes
 *
 * Included functions are:
 *	find_nodebyname() - find a node with a given name
 */
#include <pbs_config.h>   /* the master config generated by configure */

#include <stdio.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include "pbs_ifl.h"
#include "libpbs.h"
#include "list_link.h"
#include "attribute.h"
#include "credential.h"
#include "batch_request.h"
#include "server_limits.h"
#include "server.h"
#include "job.h"
#include "pbs_nodes.h"
#include "pbs_error.h"
#include "log.h"


static char ident[] = "@(#) $RCSfile: node_func.c,v $ $Revision: 2.0 $";

/* Global Data */

extern int	 svr_totnodes;
extern int	 svr_tsnodes;
extern int	 svr_clnodes;
extern char	*path_nodes_new;
extern char	*path_nodes;


/* Functions in this file
 * find_nodebyname() -      given a node name, search pbsnlist for that name
 * save_characteristic() -  save the the characteristics of the node along with
 *			    the address of the node
 * chk_characteristic() -   check for changes to the node's set of characteristics
 *			    and set appropriate flag bits in the "need_todo"
 *			    location depending on which characteristics changed
 * status_nodeattrib() -    add status of each requested (or all) node-attribute to
 *			    the status reply
 * initialize_pbsnode() -   performs node initialization on a new node
 * effective_node_delete() -  effectively deletes a node from the server's node list
 *			      by setting the node's "deleted" bit
 * setup_notification() -   sets mechanism for notifying other hosts about a new host
 * process_host_name_part() - processes hostname part of a batch request into a prop
 *			      structure, host's IP addresses into an array, and node
 *			      node type (cluster/time-shared) into an int variable
 * update_nodes_file() -    used to update the nodes file when certain changes occur
 *                          to the server's internal nodes list
 * recompute_ntype_cnts -   Recomputes the current number of cluster nodes and current
 *			    number of time-shared nodes
 */



/*
 * find_nodebyname() - find a node by its name
 */

struct pbsnode	  *find_nodebyname(nodename)
	char *nodename;
{
	int		i;
	char		*pc;
	struct pbsnode *pnode;

	pnode = (struct pbsnode *)0;
	for(i=0; i<svr_totnodes; i++ ) {
		if(pbsnlist[i]->inuse & INUSE_DELETED)
			continue;

		if( strcmp(nodename, pbsnlist[i]->last->name) == 0 ) {
		    pnode = pbsnlist[i];
		    break;
		}
	}
	return (pnode);
}




static struct pbsnode	*old_address = 0;			/*node in question */
static struct prop	*old_first = (struct prop *)0xdead;	/*node's first prop*/
static short		old_inuse = (short)0xdead;	/*node's   state   */
static short		old_ntype = (short)0xdead;	/*node's   ntype   */
static int		old_nprops = 0xdead;		/*node's   nprops  */


/*
 * save_characteristic() -  save the characteristic values of the node along
 *			    with the address of the node
 */

void	save_characteristic( pnode )
	struct pbsnode	*pnode;
{
	if( pnode == (struct pbsnode *)0 )
	    return;

	old_address = pnode;
	old_inuse =   pnode->inuse;
	old_ntype =   pnode->ntype;
	old_nprops =  pnode->nprops;
	old_first =   pnode->first;
	
}

/*
 * chk_characteristic() -  check the value of the characteristics against
 *			   that which was saved earlier.
 *			   Returns:
 *			   -1  if parent address doesn't match saved parent address
 *			    0  if successful check.  *pneed_todo gets appropriate
 *			       bit(s) set depending on the results of the check.
 *			       The "returned" bits get used by the caller.
 */


int	chk_characteristic( pnode, pneed_todo )
	struct pbsnode	*pnode;
	int		*pneed_todo;
{
	int		rc;
	short		tmp;

	if( pnode != old_address || pnode == (struct pbsnode *)0 ) {
	  /*didn't do save_characteristic() before issuing chk_characteristic()*/

	    old_address = (struct pbsnode *)0;
	    return (-1);
	}

	tmp = pnode -> inuse;
	if (tmp != old_inuse) {

	   if (tmp & INUSE_OFFLINE && !(old_inuse & INUSE_OFFLINE))
	       *pneed_todo |= WRITENODE_STATE;		/*marked offline */

	   if (!(tmp & INUSE_OFFLINE) && old_inuse & INUSE_OFFLINE)
	       *pneed_todo |= WRITENODE_STATE;		/*removed offline*/

	   if (tmp & INUSE_DELETED && old_inuse & INUSE_OFFLINE)
	       *pneed_todo |= WRITENODE_STATE;		/*offline & now deleted*/

	   if (tmp & INUSE_DELETED && !(old_inuse & INUSE_DELETED))
	       *pneed_todo |= WRITE_NEW_NODESFILE;	/*node being deleted*/
	}

	tmp = pnode -> ntype;
	if (tmp != old_ntype )
	    *pneed_todo |= WRITE_NEW_NODESFILE;

	if (old_nprops != pnode->nprops || old_first != pnode->first)
	    *pneed_todo |= WRITE_NEW_NODESFILE;

	old_address = (struct pbsnode *)0;
	return  0;
}



/* status_nodeattrib() -    add status of each requested (or all) node-attribute to
 *			    the status reply
 *
 *      Returns:     0	is success
 *                != 0	is error, if a node-attribute is incorrectly specified, *bad is
 *			set to the node-attribute's ordinal position 
 */

int status_nodeattrib(pal, padef, pnode, limit, priv, phead, bad)
        svrattrl        *pal;		/*an svrattrl from the request  */
        attribute_def   *padef;		/*the defined node attributes   */
        struct pbsnode  *pnode;		/*no longer an attribute ptr	*/
        int              limit;		/*number of array elts in padef */
        int              priv;		/*requester's privilege		*/

        list_head       *phead;		/*heads list of svrattrl structs that hang */
					/*off the brp_attr member of the status sub*/
					/*structure in the request's "reply area"  */

        int             *bad;		/*if node-attribute error record it's*/
					/*list position here                 */
{
	int   i;
	int   rc = 0;			/*return code, 0 == success*/
        int   index;
        int   nth;			/*tracks list position (ordinal tacker)   */

	attribute	atemp[ND_ATR_LAST];	/*temporary array of attributes	  */



        priv &= ATR_DFLAG_RDACC;  		/* user-client privilege          */
	
	for (i=0; i<ND_ATR_LAST; i++) {   /*set up attributes using data from node*/

	     if ( !strcmp ((padef + i)->at_name, ATTR_NODE_state) )
		  atemp[i].at_val.at_short = pnode->inuse;
	     else if ( !strcmp ((padef + i)->at_name, ATTR_NODE_properties) )
		  atemp[i].at_val.at_prop = pnode->first;
	     else if ( !strcmp ((padef + i)->at_name, ATTR_NODE_ntype) )
		  atemp[i].at_val.at_short = pnode->ntype;
	     else if ( !strcmp ((padef + i)->at_name, ATTR_NODE_jobs) )
		  atemp[i].at_val.at_jinfop = pnode->jobs;
	     else {/*we don't ever expect this*/
		  *bad = 0; 
		  return (PBSE_UNKNODEATR);
	     }
	     atemp[i].at_flags = ATR_VFLAG_SET; /*artificially set the value's flags*/
	}


        if (pal) {        /*caller has requested status on specific node-attributes*/
	    nth = 0;
            while (pal) {
                ++nth;
                index = find_attr(padef, pal->al_name, limit);
                if (index < 0) {
                        *bad = nth;		/*name in this position can't be found*/
                        rc = PBSE_UNKNODEATR;
			break;
                }
                if ((padef+index)->at_flags & priv) {
                        if ( rc = (  (padef+index)->at_encode( &atemp[index], phead,
                                        (padef+index)->at_name,
                                        (char *)0, ATR_ENCODE_CLIENT )  ) < 0  ) {
			      
			      rc = -rc;
			      break;
			}
                }
                pal = (svrattrl *)GET_NEXT(pal->al_link);
            }

        } else {        /* non-specific request, return all readable attributes    */

                for (index = 0; index < limit; index++) {
                    if (((padef+index)->at_flags & priv) &&
                        !((padef+index)->at_flags & ATR_DFLAG_NOSTAT)) {
                           if ( rc = (  (padef+index)->at_encode( &atemp[index], phead,
                                        (padef+index)->at_name,
                                        (char *)0, ATR_ENCODE_CLIENT )  ) < 0  ) {
			      
			      rc = -rc;
			      break;
			   }
                    }
                }
        }

        return (rc);
}




/*
 *	initialize_pbsnode - carries out initialization on a new
 *			     pbsnode.  The assumption is that all
 *			     the parameters are valid.
*/

void	initialize_pbsnode (pnode, pname, pul, ntype)
	struct pbsnode	*pnode;
	struct prop	*pname;	    /*prop struct with name */
	u_long		*pul;	    /*host byte order array */
				    /*ipaddrs for this node */
	int		ntype;	    /*time-shared or cluster*/
{
	pnode->jobs = (struct jobinfo *)NULL;
	pnode->flag = okay;
	pnode->nprops = 1;
	pnode->stream = -1;
	pnode->inuse = INUSE_UNKNOWN;
	pnode->ntype = ntype;
	pnode->order = 0;
	pnode->allocto = (resource_t)0;
	pnode->last = pname;
	pnode->first = pnode->last;
	pnode->addrs = pul;	    /*list of host byte order */
				    /*ip addrs's (u_long array*/
}


void	effective_node_delete (pnode)
	struct pbsnode	*pnode;
{
	struct jobinfo	*jip, *jipt;
	u_long		*up;

	pnode->last->next = (struct prop *)0; /*just in case*/
	pnode->last = (struct prop *)0;
	free_prop_list (pnode->first);
	pnode->first = (struct prop *)0;

	for (up = pnode->addrs; *up; up++) {/*del node's IP addresses frm tree*/
	    tdelete(*up, &ipaddrs);
	}
	if (pnode->addrs) {/*remove array of IP addresses*/
	    free (pnode->addrs);
	    pnode->addrs = (u_long *)0;
	}

	tdelete((u_long)pnode->stream, &streams); /*take stream out of tree*/
	(void )rpp_close (pnode->stream);
	pnode->stream = -1;

	for (jip = pnode->jobs; jip; jip = jipt) {
	       jipt = jip->next;
	       free (jip); 
	}

	pnode->jobs = (struct jobinfo *)0;
	pnode->inuse |= (INUSE_DELETED); 
	pnode->inuse &= ~(INUSE_NEEDS_HELLO_PING); 
}



/*
 *	setup_notification -  Sets up the  mechanism for notifying
 *			      other members of the server's node
 *			      pool that a new node was added manually
 *			      via qmgr.  Actual notification occurs some
 *			      time later through the ping_nodes mechanism
*/

void	setup_notification ()
{
	int	i;
	for (i=0; i<svr_totnodes; i++) {
	     if (pbsnlist[i]->inuse & INUSE_DELETED)
		 continue;
	     else
		 pbsnlist[i]->inuse |= INUSE_NEEDS_HELLO_PING;
	}
}



int	process_host_name_part (preq, pul, pname, ntype)

	struct batch_request *preq;
					/*THINGS RETURNED*/
	u_long		**pul;		/*0 terminated host adrs array*/
	struct prop	**pname;	/*node name w/o any :ts       */
	int		*ntype;		/*node type; time-shared, not */
{
	struct hostent	*hp;
	struct prop	*pprop;
	struct in_addr  addr;
	char		*phostname;     /*caller supplied hostname   */
	u_long		ipaddr;		/*ip address in host order   */
	int		len, i;


	len = strlen(preq->rq_ind.rq_manager.rq_objname);
	if (len == 0)
	    return  (PBSE_UNKNODE);

	phostname = strdup (preq->rq_ind.rq_manager.rq_objname);

	if (phostname == (char *)0 )
            return  (PBSE_SYSTEM);

	pprop = (struct prop *)malloc (sizeof(struct prop));
	if (pprop == (struct prop *)0) {
	    free (phostname);
	    return  (PBSE_SYSTEM);
	}
	
	*ntype = NTYPE_CLUSTER;
	if (len >= 3 && !strcmp (&phostname[len-3], ":ts")) {
            phostname[len-3] = '\0';
	    *ntype = NTYPE_TIMESHARED;
	}

	if ((hp = gethostbyname(phostname)) == NULL) {
	     sprintf(log_buffer, "host %s not found",
		     preq->rq_ind.rq_manager.rq_objname);
	     free (pprop);
	     free (phostname);
	     return  (PBSE_UNKNODE);
	}

	memcpy((char *)&addr, hp->h_addr, hp->h_length);
	if (hp->h_addr_list[1] == NULL) {/*weren't given cannonical name*/
	    char	*hname;

	    if ((hp = gethostbyaddr((void *)&addr,  sizeof(struct in_addr),
						hp->h_addrtype)) == NULL) {

	         sprintf(log_buffer, "addr not found h_errno=%d errno=%d",
				     h_errno, errno);
	         free (pprop);
	         free (phostname);
	         return  (PBSE_UNKNODE);
	    }
	
	    hname = (char *)strdup(hp->h_name); /*cannonical name in theory*/
	    if (hname == (char *)0) {
	        free (pprop);
	        free (phostname);
	        return  (PBSE_SYSTEM);
	    }

	    if ((hp = gethostbyname(hname)) == NULL) {
	         sprintf(log_buffer, "bad cname %s, h_errno=%d errno=%d",
				     hname, h_errno, errno);
		 free (hname);
	         free (pprop);
	         free (phostname);
	         return  (PBSE_UNKNODE);
	    }
	    free(hname);
	}

	for (i=0; hp->h_addr_list[i]; i++);	/*count how many ipadrs*/

	*pul = (u_long *)malloc(sizeof(u_long) * (i+1));  /*null end it*/
	if (*pul == (u_long *)0) {
	     free (phostname);
	     free (pprop);
	}

	for (i=0; hp->h_addr_list[i]; i++) {
	     u_long	ipaddr;

	     memcpy((char *)&addr, hp->h_addr_list[i], hp->h_length);
	     ipaddr = ntohl(addr.s_addr);
	     (*pul)[i] = ipaddr;
	}

	(*pul)[i] = 0;				/*null term array ip adrs*/
	pprop->mark = 0;
	pprop->name = phostname;
	pprop->next = (struct prop *)0;
	*pname = pprop;

	return  (0);				/*function successful    */
}



/*
 *	update_nodes_file - When called, this function will update
 *			    the nodes file.  Specifically, it will
 *			    walk the server's array of pbsnodes
 *			    constructing for each entry a nodes file
 *			    line if that entry is not marked as deleted.
 *			    These are written to a temporary file.
 *			    Upon successful conclusion that file replaces
 *			    the nodes file.
*/			    

int	update_nodes_file ()
{
	static	char	id[] = "update_nodes_file";
	struct pbsnode  *np;
	struct prop	*pp;
	char	*lp;
	int	lsize, maxLine = 0;
	int	wc, begin;
	int	i, j;
	char	*cp, *locp;
	FILE	*nin;

	DBPRT(("%s: entered\n", id))

	if ((nin = fopen(path_nodes, "r")) == NULL) {
		log_event(PBSEVENT_ADMIN, PBS_EVENTCLASS_SERVER, "nodes",
			  "No Node description file");
		return (-1);
	}
	fclose (nin);

	if ((nin = fopen(path_nodes_new, "w")) == NULL) {
		log_event(PBSEVENT_ADMIN, PBS_EVENTCLASS_SERVER, "nodes",
			  "Node description file update failed");
		return (-1);
	}

	if (svr_totnodes == 0 || pbsnlist == (struct pbsnode **)0) {
		log_event(PBSEVENT_ADMIN, PBS_EVENTCLASS_SERVER, "nodes",
			  "Server has empty nodes list");
		fclose (nin);
		return (-1);
	}

	/*compute size of largest line buffer needed*/
	for (i=0; i<svr_totnodes; ++i) {

	     np = pbsnlist[i];
	     if (np->inuse & INUSE_DELETED)
		 continue;
	     for (j=0, lsize=0, pp=np->first; j<np->nprops; j++, pp=pp->next)
		  lsize += strlen (pp->name) + 2;
	     if (maxLine < lsize)
		 maxLine = lsize;
	}

	if ( !(lp = malloc( maxLine + 2 + 3)) ) {/*+2 for newline and null*/
						 /*room for possible ":ts"*/

		log_event(PBSEVENT_ADMIN, PBS_EVENTCLASS_SERVER, "nodes",
			  "Node description file update failed");
		fclose (nin);
		return (-1);
	}


	/*for each node, build a line from the props and write it to file*/
	/*Remark: can only walk the node's prop list in forward direction*/
	/*so, the line is built from the end to the beginning in order   */
	/*that the properties in the nodes  file keep their original     */
	/*ordering on the line.  Not necessary, but it is a convenience  */

	for (i=0; i<svr_totnodes; ++i) {

	     np = pbsnlist[i];
	     if (np->inuse & INUSE_DELETED)
		 continue;

	     begin = maxLine + 4;		/*offset into buf*/
	     lp[begin--] = '\0';
	     lp[begin] = '\n';

	     for (j=0, pp=np->first; j<np->nprops - 1; j++, pp=pp->next) {

		 lsize = strlen (pp->name);
		 begin -= (lsize + 2);		/*2 spaces between props*/
		 locp = &lp[begin];
		 *locp++ = ' ',  *locp++ = ' ';
	         for (cp = pp->name; *cp ; cp++)
		     *locp++ = *cp;
	     }

	     lsize = strlen (np->last->name);
	     begin -= lsize;		/*string begins w/ node's name*/
	     if (np->ntype == NTYPE_TIMESHARED)
	         begin -= 3;

	     locp = &lp[begin];
	     for (cp = np->last->name; *cp ; cp++)
		 *locp++ = *cp;

	     if (np->ntype == NTYPE_TIMESHARED) {
		 *locp++ = ':', *locp++ = 't', *locp++ = 's';
	     }

	     lsize = strlen (&lp[begin]);
	     wc = fprintf (nin, "%s", &lp[begin]);
	     if (wc < 0 || wc != lsize) {
		log_event(PBSEVENT_ADMIN, PBS_EVENTCLASS_SERVER, "nodes",
			  "Node description file update failed");
		fflush (nin);
		fclose (nin);
		free(lp);
		return (-1);
	     }
	}
	fflush (nin);
	fclose (nin);

        (void)unlink(path_nodes);
        (void)link(path_nodes_new, path_nodes);
        (void)unlink(path_nodes_new);
	free(lp);
	return (0);
}


/*
 *	recompute_ntype_cnts - Recomputes the current number of cluster
 *			       nodes and current number of time-shared nodes
 */
void	recompute_ntype_cnts ()
{
	int		 svr_loc_clnodes = 0;
	int		 svr_loc_tsnodes = 0;
	int		 i;
	struct	pbsnode  *pnode;

	if (svr_totnodes) {
	    for (i=0; i<svr_totnodes; ++i) {

		 pnode = pbsnlist[i];
		 if (pnode->inuse & INUSE_DELETED)
		     continue;

		 if (pnode->ntype == NTYPE_CLUSTER)
		     svr_loc_clnodes++;
		 else if (pnode->ntype == NTYPE_TIMESHARED)
		     svr_loc_tsnodes++;
	    }
	    svr_clnodes = svr_loc_clnodes;
	    svr_tsnodes = svr_loc_tsnodes;
	}
}
