/*
This program is confidential and proprietary to MRJ Technology Solutions
and may not be reproduced, published or disclosed to others without
written authorization from MRJ.

Copyright (c) 1998 MRJ Technology Solutions, Inc. All Rights Reserved.

This program is derived from prior work some of which was originally
written as a joint project between the Numerical Aerospace Simulation
(NAS) Systems Division of NASA Ames Research Center and the National
Energy Research Supercomputer Center (NERSC) of Lawrence Livermore
National Laboratory.
*/
/*
 * req_message.c - functions dealing with sending a message 
 *		     to a running job.
 */

#include <pbs_config.h>   /* the master config generated by configure */

#include <sys/types.h>
#include "libpbs.h"
#include <signal.h>
#include "server_limits.h"
#include "list_link.h"
#include "attribute.h"
#include "server.h"
#include "credential.h"
#include "batch_request.h"
#include "job.h"
#include "work_task.h"
#include "pbs_error.h"
#include "log.h"
#include "svrfunc.h"

static char ident[] = "@(#) $RCSfile: req_message.c,v $ $Revision: 2.0 $";

/* Private Function local to this file */

static void post_message_req A_((struct work_task *));

/* Global Data Items: */

extern int   pbs_mom_port;
extern char *msg_messagejob;


/*
 * req_messagejob - service the Message Job Request
 *
 *	This request sends (via MOM) a message to a running job.
 */

void req_messagejob(preq)
	struct batch_request *preq;
{
	job		 *pjob;
	int		  rc;

	if ((pjob = chk_job_request(preq->rq_ind.rq_message.rq_jid, preq)) == 0)
		return;

	/* the job must be running */

	if (pjob->ji_qs.ji_state != JOB_STATE_RUNNING) {
		req_reject(PBSE_BADSTATE, 0, preq);
		return;
	}
	
	/* pass the request on to MOM */

	if (rc = relay_to_mom(pjob->ji_qs.ji_un.ji_exect.ji_momaddr,
			      preq, post_message_req))
		req_reject(rc, 0, preq);	/* unable to get to MOM */

	/* After MOM acts and replies to us, we pick up in post_message_req() */
}

/*
 * post_message_req - complete a Message Job Request
 */

static void post_message_req(pwt)
	struct work_task *pwt;
{
	struct batch_request *preq;

	svr_disconnect(pwt->wt_event);	/* close connection to MOM */
	preq = pwt->wt_parm1;
	preq->rq_conn = preq->rq_orgconn;  /* restore socket to client */

	(void)sprintf(log_buffer, msg_messagejob, preq->rq_reply.brp_code);
	log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, 
		  preq->rq_ind.rq_message.rq_jid, log_buffer);
	if (preq->rq_reply.brp_code)
		req_reject(preq->rq_reply.brp_code, 0, preq);
	else
		reply_ack(preq);
}
