/*
This program is confidential and proprietary to MRJ Technology Solutions
and may not be reproduced, published or disclosed to others without
written authorization from MRJ.

Copyright (c) 1998 MRJ Technology Solutions, Inc. All Rights Reserved.

This program is derived from prior work some of which was originally
written as a joint project between the Numerical Aerospace Simulation
(NAS) Systems Division of NASA Ames Research Center and the National
Energy Research Supercomputer Center (NERSC) of Lawrence Livermore
National Laboratory.
*/

#include <pbs_config.h>   /* the master config generated by configure */

/*
 * svr_resc_def is the array of resource definitions for the server.
 * Each legal server resource is defined here.
 */

/*
 * The entries for each resource definition are (see resource.h):
 *	name,
 *	decode function,
 *	encode function,
 *	set function,
 *	compare function,
 *	free value space function,
 *	access permission flags,
 *	value type
 */

    {	"ncpus",			/* number of processors for job */
	decode_l,
	encode_l,
	set_l,
	comp_l,
	free_null,
	NULL_FUNC,
	READ_WRITE | ATR_DFLAG_MOM | ATR_DFLAG_RASSN,
	ATR_TYPE_LONG
    },
    {	"cput",				/* total cput time for job */
	decode_time,
	encode_time,
	set_l,
	comp_l,
	free_null,
	NULL_FUNC,
	READ_WRITE | ATR_DFLAG_MOM | ATR_DFLAG_ALTRUN,
	ATR_TYPE_LONG
    },
    {	"pcput",			/* max cput time for any process */ 
	decode_time,
	encode_time,
	set_l,
	comp_l,
	free_null,
	NULL_FUNC,
	READ_WRITE | ATR_DFLAG_MOM | ATR_DFLAG_ALTRUN,
	ATR_TYPE_LONG
    },
    {	"mem",				/* max memory for job */
	decode_size,
        encode_size,
        set_size,
        comp_size,
        free_null,
	NULL_FUNC,
        READ_WRITE | ATR_DFLAG_MOM | ATR_DFLAG_ALTRUN | ATR_DFLAG_RASSN,
        ATR_TYPE_SIZE
    },
    {	"pmem",				/* max memory per process */
	decode_size,
        encode_size,
        set_size,
        comp_size,
        free_null,
	NULL_FUNC,
        READ_WRITE | ATR_DFLAG_MOM | ATR_DFLAG_ALTRUN,
        ATR_TYPE_SIZE
    },
    {	"mta",				/* mag tape, a class */
	decode_l,
	encode_l,
	set_l,
	comp_l,
	free_null,
	NULL_FUNC,
	READ_WRITE | ATR_DFLAG_MOM,
	ATR_TYPE_LONG
    },
    {	"mtb",				/* mag tape, b class */
	decode_l,
	encode_l,
	set_l,
	comp_l,
	free_null,
	NULL_FUNC,
	READ_WRITE | ATR_DFLAG_MOM,
	ATR_TYPE_LONG
    },
    {	"mtc",				/* mag tape, c class */
	decode_l,
	encode_l,
	set_l,
	comp_l,
	free_null,
	NULL_FUNC,
	READ_WRITE | ATR_DFLAG_MOM,
	ATR_TYPE_LONG
    },
    {	"mtd",				/* mag tape, d class */
	decode_l,
	encode_l,
	set_l,
	comp_l,
	free_null,
	NULL_FUNC,
	READ_WRITE | ATR_DFLAG_MOM,
	ATR_TYPE_LONG
    },
    {	"mte",				/* mag tape, e class */
	decode_l,
	encode_l,
	set_l,
	comp_l,
	free_null,
	NULL_FUNC,
	READ_WRITE | ATR_DFLAG_MOM,
	ATR_TYPE_LONG
    },
    {	"mtf",				/* mag tape, f class */
	decode_l,
	encode_l,
	set_l,
	comp_l,
	free_null,
	NULL_FUNC,
	READ_WRITE | ATR_DFLAG_MOM,
	ATR_TYPE_LONG
    },
    {	"mtg",				/* mag tape, g class */
	decode_l,
	encode_l,
	set_l,
	comp_l,
	free_null,
	NULL_FUNC,
	READ_WRITE | ATR_DFLAG_MOM,
	ATR_TYPE_LONG
    },
    {	"mth",				/* mag tape, h class */
	decode_l,
	encode_l,
	set_l,
	comp_l,
	free_null,
	NULL_FUNC,
	READ_WRITE | ATR_DFLAG_MOM,
	ATR_TYPE_LONG
    },
    {	"pf",				/* max file space for job */
	decode_size,
        encode_size,
        set_size,
        comp_size,
        free_null,
	NULL_FUNC,
        READ_WRITE | ATR_DFLAG_MOM | ATR_DFLAG_ALTRUN,
        ATR_TYPE_SIZE
    },
    {	"ppf",				/* max file space per process */
	decode_size,
        encode_size,
        set_size,
        comp_size,
        free_null,
	NULL_FUNC,
        READ_WRITE | ATR_DFLAG_MOM | ATR_DFLAG_ALTRUN,
        ATR_TYPE_SIZE
    },
    {	"sds",				/* max SDS for job */
	decode_size,
        encode_size,
        set_size,
        comp_size,
        free_null,
	NULL_FUNC,
        READ_WRITE | ATR_DFLAG_MOM | ATR_DFLAG_ALTRUN | ATR_DFLAG_RASSN,
        ATR_TYPE_SIZE
    },
    {	"psds",				/* max SDS per process */
	decode_size,
        encode_size,
        set_size,
        comp_size,
        free_null,
	NULL_FUNC,
        READ_WRITE | ATR_DFLAG_MOM | ATR_DFLAG_ALTRUN,
        ATR_TYPE_SIZE
    },
    {	"procs",			/* number of processes per job */
	decode_l,
	encode_l,
	set_l,
	comp_l,
	free_null,
	NULL_FUNC,
	READ_WRITE | ATR_DFLAG_MOM | ATR_DFLAG_RASSN,
	ATR_TYPE_LONG
    },
    {	"mppe",				/* number of mpp nodes */
	decode_l,
	encode_l,
	set_l,
	comp_l,
	free_null,
	NULL_FUNC,
	READ_WRITE | ATR_DFLAG_MOM | ATR_DFLAG_RASSN,
	ATR_TYPE_LONG
    },
    {	"mppt",				/* total mpp time for job */
	decode_time,
	encode_time,
	set_l,
	comp_l,
	free_null,
	NULL_FUNC,
	READ_WRITE | ATR_DFLAG_MOM | ATR_DFLAG_ALTRUN,
	ATR_TYPE_LONG
    },
    {	"pmppt",			/* max mpp time for any process */ 
	decode_time,
	encode_time,
	set_l,
	comp_l,
	free_null,
	NULL_FUNC,
	READ_WRITE | ATR_DFLAG_MOM | ATR_DFLAG_ALTRUN,
	ATR_TYPE_LONG
    },
    {	"nice",				/* job nice value */
	decode_l,
	encode_l,
	set_l,
	comp_l,
	free_null,
	NULL_FUNC,
	READ_WRITE | ATR_DFLAG_MOM,
	ATR_TYPE_LONG
    },
    {	"walltime",			/* wall clock time limit for a job */
	decode_time,
	encode_time,
	set_l,
	comp_l,
	free_null,
	NULL_FUNC,
	READ_WRITE | ATR_DFLAG_MOM | ATR_DFLAG_ALTRUN,
	ATR_TYPE_LONG
    },
#if	SRFS
    {	"srfs_tmp",			/* SRFS TMPDIR allocation */
	decode_size,
        encode_size,
        set_size,
        comp_size,
        free_null,
	NULL_FUNC,
        READ_WRITE | ATR_DFLAG_MOM,
        ATR_TYPE_SIZE
    },
    {	"srfs_big",			/* SRFS BIGDIR allocation */
	decode_size,
        encode_size,
        set_size,
        comp_size,
        free_null,
	NULL_FUNC,
        READ_WRITE | ATR_DFLAG_MOM,
        ATR_TYPE_SIZE
    },
    {	"srfs_fast",			/* SRFS FASTDIR allocation */
	decode_size,
        encode_size,
        set_size,
        comp_size,
        free_null,
	NULL_FUNC,
        READ_WRITE | ATR_DFLAG_MOM,
        ATR_TYPE_SIZE
    },
    {	"srfs_assist",			/* SRFS assist flag */
	decode_b,
	encode_b,
	set_b,
	comp_b,
	free_null,
	NULL_FUNC,
	ATR_DFLAG_OPRD | ATR_DFLAG_MGRD | ATR_DFLAG_SvRD | ATR_DFLAG_SvWR | ATR_DFLAG_MOM,
	ATR_TYPE_LONG
    },
#endif	/* SRFS */
